#!/usr/bin/env python

# Sample command line usage:
#   python stretch_realsense_jog.py --save stretch_video_max60minutes.avi
#   python stretch_realsense_jog.py --save stretch_video_max90minutes.avi --save_limit 90
# See the parser arguments below for more command-line options.
# See the below code for more settings such as resolutions and whether to use color and/or depth images.
#
# Note that the actual frame rate may be lower than the nominal frame rate, so playback of saved videos
#  may be at the wrong speed.  This is especially relevant when using higher image resolutions.
#  The real-time duration and actual average frame rate will be added to the saved filenames,
#  so post-processing can try to speed up the videos. However, the actual frame rate is often variable.
#
# Originally posted: 2021-05-17 by Binit Shah (see https://forum.hello-robot.com/t/170)
# Revised:           2021-05-30 by Joseph DelPreto

import pyrealsense2 as rs
import numpy as np
import cv2

import time, datetime
import os
import pathlib
import argparse
import stretch_body.hello_utils as hu
hu.print_stretch_re_use()

######################################
# Settings via command line arguments
######################################

parser = argparse.ArgumentParser(description='Tool to test the Realsense D435i Camera.')
parser.add_argument("--no_gui", action="store_true",
                    help="Show no GUI while reading images.")
parser.add_argument("--colormap", type=int, default=cv2.COLORMAP_OCEAN,
                    help="Valid OpenCV colormaps at 'https://docs.opencv.org/master/d3/d50/group__imgproc__colormap.html'.")
parser.add_argument("--save", nargs="?", type=str, const="",
                    help="Save as .avi video to given filepath at end of script.")
parser.add_argument("--save_limit", type=int, default=60,
                    help="The number of minutes of data to save.")
args, _ = parser.parse_known_args()
output_filepath = args.save

####################################
# Additional settings
####################################

# Whether to save a video of the color images and/or depth images.
save_video_color = True
save_video_depth = True
save_video_colorAndDepth = True # color and depth concatenated side-by-side
# Video streaming configuration.
# Note: the actual fps will probably be lower than the target, especially if larger resolutions are used or multiple videos are saved.
fps_color = 30 # FPS for color-only videos and for color+depth videos
fps_depth_downsample_factor = 3 # The depth frame rate will be fps_color/fps_depth_downsample_factor.
                                # Only used for the depth-only video stream (combined color+depth videos will be at the color fps).
resolution_color = [640, 480]   # [1920, 1080], [1280, 720], [640, 480]
resolution_depth = [640, 480]   # [1280, 720], [640, 480]
frameBuffer_limit_s = 10 # How many seconds of frames to keep in the buffer - probably something short to avoid memory usage issues
                         # (different from save_limit, which limits the overall video capture length and is set by the command line argument above).
# How to adjust the filenames (to add more info and to avoid accidentally overwriting videos).
output_filepath_addTimestamp = True
output_filepath_addFps = True
output_filepath_addDuration = True
# Some image processing options.
apply_local_histogram_equalization = False
apply_global_histogram_equalization = False

####################################
# Initialize
####################################

# Configure depth and color streams.
# Note: width/heights are swapped since the images will be rotated 90 degrees from what the camera captures.
pipeline = rs.pipeline()
config = rs.config()
config.enable_stream(rs.stream.depth, resolution_depth[0], resolution_depth[1], rs.format.z16, fps_color) # note that the fps downsampling will be applied later
config.enable_stream(rs.stream.color, resolution_color[0], resolution_color[1], rs.format.bgr8, fps_color)
frame_width_colorAndDepth = resolution_color[1] + resolution_depth[1]
frame_height_colorAndDepth = max(resolution_color[0], resolution_depth[0])
frame_width_color = resolution_color[1]
frame_height_color = resolution_color[0]
frame_width_depth = resolution_depth[1]
frame_height_depth = resolution_depth[0]

# Create writer(s) to save the stream(s) if desired.
writer_color = None
writer_depth = None
writer_colorAndDepth = None
if (output_filepath is not None) and (os.access(str(pathlib.Path(output_filepath).parent), os.W_OK)):
    # Add a timestamp to the filename if desired.
    if output_filepath_addTimestamp:
        date_str = datetime.datetime.now().strftime("%Y-%m-%d_%H-%M-%S")
        (dir, filename) = os.path.split(output_filepath)
        (filename_base, ext) = os.path.splitext(os.path.basename(output_filepath))
        output_filepath = os.path.join(dir, '%s_%s%s' % (date_str, filename_base, ext))
    # Strip the extension if one was provided so we can ensure it's an AVI file.
    (dir, filename) = os.path.split(output_filepath)
    (filename_base, ext) = os.path.splitext(os.path.basename(output_filepath))
    # Indicate whether it's color and/or depth images.
    output_filepath_color = os.path.join(dir, '%s_color.avi' % (filename_base))
    output_filepath_depth = os.path.join(dir, '%s_depth.avi' % (filename_base))
    output_filepath_colorAndDepth = os.path.join(dir, '%s_colorAndDepth.avi' % (filename_base))
    # Open the writer(s).
    if save_video_color:
        writer_color = cv2.VideoWriter(str(pathlib.Path(output_filepath_color)), cv2.VideoWriter_fourcc(*'MJPG'), fps_color, (frame_width_color, frame_height_color))
    if save_video_depth:
        writer_depth = cv2.VideoWriter(str(pathlib.Path(output_filepath_depth)), cv2.VideoWriter_fourcc(*'MJPG'), fps_color/fps_depth_downsample_factor, (frame_width_depth, frame_height_depth))
    if save_video_colorAndDepth:
        writer_colorAndDepth = cv2.VideoWriter(str(pathlib.Path(output_filepath_colorAndDepth)), cv2.VideoWriter_fourcc(*'MJPG'), fps_color, (frame_width_colorAndDepth, frame_height_colorAndDepth))

# Check if no display exists.
try:
    if hu.get_display() is None:
        if args.no_gui is False:
            print('No display found. Setting no_gui=true')
        args.no_gui = True
except:
    args.no_gui = True

####################################
# Stream!
####################################

frameBuffer_color = []
frameBuffer_depth = []
frameBuffer_colorAndDepth = []
frameBuffer_limit_color = round(frameBuffer_limit_s * fps_color)
frameBuffer_limit_depth = round(frameBuffer_limit_s * fps_color / fps_depth_downsample_factor)
capture_limit_color = round(fps_color * 60 * args.save_limit)

pipeline.start(config)
start_capture_time_s = time.time()
end_capture_time_color_s = start_capture_time_s
end_capture_time_depth_s = start_capture_time_s
frame_count_color = 0
frame_count_depth = 0
print("Press Ctrl-C to exit...")
try:
    while frame_count_color < capture_limit_color:
        # Get the latest frames from the camera.
        frames = pipeline.wait_for_frames()
        depth_frame = frames.get_depth_frame()
        color_frame = frames.get_color_frame()
        if not depth_frame or not color_frame:
            continue
        capture_time_s = time.time()

        # Convert images to numpy arrays.
        depth_image = np.asanyarray(depth_frame.get_data())
        color_image = np.asanyarray(color_frame.get_data())

        # Apply histogram equalization if desired.
        if apply_local_histogram_equalization:
            clahe = cv2.createCLAHE(clipLimit=2.0, tileGridSize=(8,8))
            color_image_lab = cv2.cvtColor(color_image, cv2.COLOR_BGR2LAB)  # convert from BGR to LAB color space
            color_image_l, color_image_a, color_image_b = cv2.split(color_image_lab)  # split on 3 different channels
            color_image_l_equalized = clahe.apply(color_image_l)  # apply CLAHE to the L-channel
            color_image_lab = cv2.merge((color_image_l_equalized, color_image_a, color_image_b))  # merge channels
            color_image = cv2.cvtColor(color_image_lab, cv2.COLOR_LAB2BGR)  # convert from LAB to BGR
        elif apply_global_histogram_equalization:
            color_image_lab = cv2.cvtColor(color_image, cv2.COLOR_BGR2LAB)  # convert from BGR to LAB color space
            color_image_l, color_image_a, color_image_b = cv2.split(color_image_lab)  # split on 3 different channels
            color_image_l_equalized = cv2.equalizeHist(color_image_l) # apply global equalization to the L-channel
            color_image_lab = cv2.merge((color_image_l_equalized, color_image_a, color_image_b))  # merge channels
            color_image = cv2.cvtColor(color_image_lab, cv2.COLOR_LAB2BGR)  # convert from LAB to BGR

        # Apply colormap on depth image (image must be converted to 8-bit per pixel first).
        depth_image = cv2.applyColorMap(cv2.convertScaleAbs(depth_image, alpha=-0.04, beta=255.0), args.colormap)

        # Rotate and flip images.
        color_image = np.moveaxis(color_image, 0, 1)
        color_image = np.fliplr(color_image)
        depth_image = np.moveaxis(depth_image, 0, 1)
        depth_image = np.fliplr(depth_image)

        # Concatenate images horizontally if desired.
        if save_video_colorAndDepth or not args.no_gui:
            # Pad images if they are different heights
            pad_y = color_image.shape[0] - depth_image.shape[0]
            if pad_y > 0:
                color_image_padded = color_image
                depth_image_padded = np.pad(depth_image, ((pad_y, 0), (0, 0), (0, 0)), mode='constant', constant_values=0)
            else:
                pad_y = abs(pad_y)
                color_image_padded = np.pad(color_image, ((pad_y, 0), (0, 0), (0, 0)), mode='constant', constant_values=0)
                depth_image_padded = depth_image
            # Tile the images
            colorAndDepth_image = np.hstack((color_image_padded, depth_image_padded))
        else:
            colorAndDepth_image = None

        # Show stream if no_gui disabled.
        if not args.no_gui:
            cv2.namedWindow('Realsense', cv2.WINDOW_AUTOSIZE)
            cv2.imshow('Realsense', colorAndDepth_image)
            if cv2.waitKey(1) & 0xFF != 255:
                raise KeyboardInterrupt()

        # Maintain stream cache.
        # Important to use copy() here depending on how the stream is used.
        frameBuffer_color.append(color_image.copy())
        frameBuffer_depth.append(depth_image.copy())
        frameBuffer_colorAndDepth.append(colorAndDepth_image)
        if len(frameBuffer_color) > frameBuffer_limit_color:
            frameBuffer_color.pop(0)
            frameBuffer_colorAndDepth.pop(0)
        if len(frameBuffer_depth) > frameBuffer_limit_depth:
            frameBuffer_depth.pop(0)
        # Write to video output(s) if desired.
        if writer_color is not None:
            writer_color.write(frameBuffer_color[-1])
            end_capture_time_color_s = capture_time_s
        if writer_colorAndDepth is not None:
            writer_colorAndDepth.write(frameBuffer_colorAndDepth[-1])
            end_capture_time_color_s = capture_time_s
        if writer_depth is not None and frame_count_color % fps_depth_downsample_factor == 0:
            writer_depth.write(frameBuffer_depth[-1])
            end_capture_time_depth_s = capture_time_s
            frame_count_depth += 1
        frame_count_color += 1
except KeyboardInterrupt:
    pass
finally:
    ####################################
    # Stop video streaming
    ####################################

    # Stop the streaming pipeline
    pipeline.stop()
    # Stop the video writer(s)
    if writer_color is not None:
        writer_color.release()
    if writer_colorAndDepth is not None:
        writer_colorAndDepth.release()
    if writer_depth is not None:
        writer_depth.release()

####################################
# Clean up and finish saving videos
####################################

# Get some information about the videos
capture_duration_color_s = end_capture_time_color_s - start_capture_time_s
capture_duration_depth_s = end_capture_time_depth_s - start_capture_time_s
capture_fps_color = (float(frame_count_color)/float(capture_duration_color_s)) if capture_duration_color_s > 0 else None
capture_fps_depth = (float(frame_count_depth)/float(capture_duration_depth_s)) if capture_duration_depth_s > 0 else None

# Add additional information to the filenames if desired
def addDurationToFilename(filepath, duration_s):
    duration_str = ('%0.3f' % duration_s)
    duration_str_forFilename = duration_str.replace('.', '-')
    (dir, filename) = os.path.split(filepath)
    (filename_base, ext) = os.path.splitext(os.path.basename(filepath))
    filepath_withDuration = os.path.join(dir, '%s_duration%ss%s' % (filename_base, duration_str_forFilename, ext))
    os.rename(filepath, filepath_withDuration)
    return filepath_withDuration
def addFpsToFilename(filepath, fps):
    fps_str = ('%0.3f' % fps)
    fps_str_forFilename = fps_str.replace('.', '-')
    (dir, filename) = os.path.split(filepath)
    (filename_base, ext) = os.path.splitext(os.path.basename(filepath))
    filepath_withFps = os.path.join(dir, '%s_fps%s%s' % (filename_base, fps_str_forFilename, ext))
    os.rename(filepath, filepath_withFps)
    return filepath_withFps
if output_filepath_addDuration:
    if writer_color is not None and os.path.exists(output_filepath_color):
        output_filepath_color = addDurationToFilename(output_filepath_color, capture_duration_color_s)
    if writer_colorAndDepth is not None and os.path.exists(output_filepath_colorAndDepth):
        output_filepath_colorAndDepth = addDurationToFilename(output_filepath_colorAndDepth, capture_duration_color_s)
    if writer_depth is not None and os.path.exists(output_filepath_depth):
        output_filepath_depth = addDurationToFilename(output_filepath_depth, capture_duration_depth_s)
if output_filepath_addFps:
    if writer_color is not None and os.path.exists(output_filepath_color):
        output_filepath_color = addFpsToFilename(output_filepath_color, capture_fps_color)
    if writer_colorAndDepth is not None and os.path.exists(output_filepath_colorAndDepth):
        output_filepath_colorAndDepth = addFpsToFilename(output_filepath_colorAndDepth, capture_fps_color)
    if writer_depth is not None and os.path.exists(output_filepath_depth):
        output_filepath_depth = addFpsToFilename(output_filepath_depth, capture_fps_depth)

print('')
print('')
if capture_fps_color is not None:
    print('Captured %d color frames in %0.3f seconds' % (frame_count_color, capture_duration_color_s))
    print('(average FPS: %8.5f)' % capture_fps_color)
if capture_fps_depth is not None:
    print('Captured %d depth frames in %0.3f seconds' % (frame_count_depth, capture_duration_depth_s))
    print('(average FPS: %8.5f)' % capture_fps_depth)
print('')
if writer_color is not None:
    print('Saved color video at %g fps to %s' % (fps_color, output_filepath_color))
if writer_depth is not None:
    print('Saved depth video at %g fps to %s' % (fps_color/fps_depth_downsample_factor, output_filepath_depth))
if writer_colorAndDepth is not None:
    print('Saved color+depth video at %g fps to %s' % (fps_color, output_filepath_colorAndDepth))
if writer_color is not None or writer_depth is not None or writer_colorAndDepth is not None:
    print('')
    print('* Note that the actual frame rates are different than the frame rates ')
    print('  of the saved videos, so playback speed may be inaccurate.')
print('')



#!/usr/bin/env python
from __future__ import print_function
import stretch_body.pimu as pimu
from colorama import Fore, Back, Style
import argparse
import stretch_body.hello_utils as hu
hu.print_stretch_re_use()

parser=argparse.ArgumentParser(description='Print battery state to console')
args=parser.parse_args()


def val_in_range(val_name, val,vmin, vmax):
    p=val <=vmax and val>=vmin
    if p:
        print(Fore.GREEN +'[Pass] ' + val_name + ' with ' + str(val))
    else:
        print(Fore.RED +'[Fail] ' + val_name + ' with ' +str(val)+ ' out of range ' +str(vmin) + ' to ' + str(vmax))

# #####################################################

p=pimu.Pimu()
if not p.startup():
    exit()
p.pull_status()
val_in_range('Voltage',p.status['voltage'], vmin=p.config['low_voltage_alert'], vmax=14.0)
val_in_range('Current',p.status['current'], vmin=0.1, vmax=p.config['high_current_alert'])
val_in_range('CPU Temp',p.status['cpu_temp'], vmin=15, vmax=80)
print(Style.RESET_ALL)
p.stop()



#!/usr/bin/env python
from __future__ import print_function
import sys, tty, termios
import time
from stretch_body.hello_utils import *
import stretch_body.base as base
import stretch_body.pimu as pimu
import argparse

print_stretch_re_use()

parser=argparse.ArgumentParser(description='Jog the base motion from the keyboard')
args=parser.parse_args()

small_rotate_rad=deg_to_rad(2.0)
large_rotate_rad=deg_to_rad(20.0)

b=base.Base()
if not b.startup(threaded=False):
    exit()
large_move_m=0.1
small_move_m=large_move_m/8

v_r_slow = b.translation_to_rotation(b.params['motion']['slow']['vel_m'])
v_r_def = b.translation_to_rotation(b.params['motion']['default']['vel_m'])
v_r_fast=b.translation_to_rotation(b.params['motion']['fast']['vel_m'])
v_r_max = b.translation_to_rotation(b.params['motion']['max']['vel_m'])
a_r_slow = b.translation_to_rotation(b.params['motion']['slow']['accel_m'])
a_r_def = b.translation_to_rotation(b.params['motion']['default']['accel_m'])
a_r_fast=b.translation_to_rotation(b.params['motion']['fast']['accel_m'])
a_r_max = b.translation_to_rotation(b.params['motion']['max']['accel_m'])
v_r={'fast':v_r_fast,'default':v_r_def,'slow':v_r_slow,'max':v_r_max}
a_r={'fast':a_r_fast,'default':a_r_def,'slow':a_r_slow,'max':a_r_max}

p=pimu.Pimu()
p.startup()

def get_keystroke():

    fd=sys.stdin.fileno()
    old_settings=termios.tcgetattr(fd)
    try:
        tty.setraw(sys.stdin.fileno())
        ch=sys.stdin.read(1)
    finally:
        termios.tcsetattr(fd,termios.TCSADRAIN,old_settings)
    return ch

def menu():
    print('--------------')
    print('m: menu')
    print('')
    print('1: rate slow')
    print('2: rate default')
    print('3: rate fast')
    print('4: rate max')
    print('w: CW/CCW 90 deg')
    print('x: forward-> back 0.5m')
    print('y: spin at 22.5deg/s')
    print('')
    print('f / b / l / r : small forward / back / left / right')
    print('F / B / L / R : large forward / back / left / right')
    print('g: enable guarded contact')
    print('d: disable guarded contact')
    print('o: freewheel')
    print('p: pretty print')
    print('q: quit')
    print('')
    print('Input?')

rate ='default'

try:
    menu()
    while True:
        if True:

            c=get_keystroke()

            #Read current motor positions when in sync mode
            #p.trigger_motor_sync()
            #time.sleep(0.1)
            b.pull_status()
            #print('################################'
            b.pretty_print()
            if c=='g':
                b.left_wheel.enable_guarded_mode()
                b.right_wheel.enable_guarded_mode()
            if c=='d':
                b.left_wheel.disable_guarded_mode()
                b.right_wheel.disable_guarded_mode()
            if c=='p':
                b.pretty_print()
            if c == '1':
                rate = 'slow'
            if c == '2':
                rate = 'default'
            if c == '3':
                rate = 'fast'
            if c == '4':
                rate = 'max'
            if c =='x':
                    print('Enter pause before starting motion (s)[10]')
                    try:
                        x=float(input())
                    except ValueError:
                        x=10.0
                    time.sleep(x)
                    b.translate_by(x_m=0.5, v_m=b.params['motion'][rate]['vel_m'], a_m=b.params['motion'][rate]['accel_m'])
                    b.push_command()
                    p.trigger_motor_sync()
                    time.sleep(4.0)
                    b.translate_by(x_m=-0.5, v_m=b.params['motion'][rate]['vel_m'], a_m=b.params['motion'][rate]['accel_m'])
                    b.push_command()
                    p.trigger_motor_sync()
                    time.sleep(4.0)
            if c =='w':
                print('Enter pause before starting motion (s)[10]')
                try:
                    x = float(input())
                except ValueError:
                    x = 10.0
                time.sleep(x)
                b.rotate_by(x_r=deg_to_rad(90.0), v_r=v_r[rate], a_r=a_r[rate])
                b.push_command()
                p.trigger_motor_sync()
                time.sleep(4.0)
                b.rotate_by(x_r=deg_to_rad(-90.0), v_r=v_r[rate], a_r=a_r[rate])
                b.push_command()
                p.trigger_motor_sync()
                time.sleep(4.0)
            if c =='y':
                print('Enter pause before starting motion (s)[10]')
                try:
                    x = float(input())
                except ValueError:
                    x = 10.0
                time.sleep(x)
                b.set_rotational_velocity(v_r=deg_to_rad(90.0)/4.0)
                b.push_command()
                p.trigger_motor_sync()
                ts=time.time()
                print('Starting 30s turn at 90deg/sec')
                tsl=time.time()
                while time.time()-ts<30.0:
                    b.pull_status()
                    print('DT:',time.time()-ts,'Vel (deg/s)', rad_to_deg(b.status['theta_vel']))
                    time.sleep(0.5)
                b.set_rotational_velocity(v_r=0)
                b.push_command()
                p.trigger_motor_sync()
                menu()
            if c == 'f':
                b.translate_by(small_move_m, v_m=b.params['motion'][rate]['vel_m'], a_m=b.params['motion'][rate]['accel_m'])
            if c == 'b':
                b.translate_by(-1*small_move_m, v_m=b.params['motion'][rate]['vel_m'], a_m=b.params['motion'][rate]['accel_m'])
            if c == 'l':
                b.rotate_by(small_rotate_rad, v_r=v_r[rate], a_r=a_r[rate])
            if c == 'r':
                b.rotate_by(-1*small_rotate_rad, v_r=v_r[rate], a_r=a_r[rate])
            if c == 'F':
                b.translate_by(large_move_m, v_m=b.params['motion'][rate]['vel_m'], a_m=b.params['motion'][rate]['accel_m'])
            if c == 'B':
                b.translate_by(-1*large_move_m, v_m=b.params['motion'][rate]['vel_m'], a_m=b.params['motion'][rate]['accel_m'])
            if c == 'L':
                b.rotate_by(large_rotate_rad, v_r=v_r[rate], a_r=a_r[rate])
            if c == 'R':
                b.rotate_by(-1*large_rotate_rad, v_r=v_r[rate], a_r=a_r[rate])
            if c == 'o':
                b.enable_freewheel_mode()
            if c=='m':
                menu()
            if c=="Q" or c=='q':
                break

            b.push_command()
            p.trigger_motor_sync()
            time.sleep(0.1)
except (KeyboardInterrupt, SystemExit):
    pass
b.stop()



#!/usr/bin/env python
import os
import yaml

params_filename = os.environ['HELLO_FLEET_PATH']+'/'+os.environ['HELLO_FLEET_ID']+'/stretch_configuration_params.yaml'

with open(params_filename, 'r') as fid:
    params = yaml.safe_load(fid)
    robot_info = params['robot']
    batch_name = robot_info['batch_name']
    serial_number = robot_info['serial_no']
    model=robot_info['model_name']

batch_name_string = 'Stretch Batch Name: {0}'.format(batch_name)
serial_number_string = 'Stretch Serial Number: {0}'.format(serial_number)
model_string = 'Stretch Model: {0}'.format(model)

print(batch_name_string)
print(serial_number_string)
print(model_string)



#!/usr/bin/env python
from __future__ import print_function
import sys
from stretch_body.wacc import Wacc
import argparse
import stretch_body.hello_utils as hu
hu.print_stretch_re_use()

parser=argparse.ArgumentParser(description='Comnmand and query the Wacc (Wrist Accelerometer) board from the keyboard')
args=parser.parse_args()

w=Wacc()
if not w.startup():
    exit()



def menu():
    print('------ MENU -------')
    print('m: menu')
    print('r: reset board')
    print('a: set D2 on')
    print('b: set D2 off')
    print('c: set D3 on')
    print('d: set D3 off')
    print('-------------------')

def step_interaction():
    menu()
    x=sys.stdin.readline()
    if len(x)>1:
        if x[0]=='m':
            menu()
        if x[0] == 'a':
            w.set_D2(1)
        if x[0] == 'b':
            w.set_D2(0)
        if x[0] == 'c':
            w.set_D3(1)
        if x[0] == 'd':
            w.set_D3(0)
        if x[0]=='r':
            print('Resetting Board. Exiting...')
            w.board_reset()
            w.push_command()
            exit()
        w.push_command()
    else:
        w.pull_status()
        w.pretty_print()

try:
    while True:
        try:
            step_interaction()
        except (ValueError):
            print('Bad input...')
except (KeyboardInterrupt, SystemExit):
    w.stop()



#!/usr/bin/env python
from __future__ import print_function
import stretch_body.robot as robot
import argparse
import stretch_body.hello_utils as hu
hu.print_stretch_re_use()

parser=argparse.ArgumentParser(description='Find zeros for all robot joints')
args=parser.parse_args()

r=robot.Robot()
r.startup()
if not r.pimu.status['runstop_event']:
    r.home()
else:
    r.logger.warning('Cannot home while run-stopped')
r.stop()



#!/usr/bin/env python
from __future__ import print_function
import os
import subprocess
import stretch_body.hello_utils as hu
hu.print_stretch_re_use()

print('Versions')
print("-------------------------------------------------------------------------")
import stretch_body
import stretch_body.base as base
import stretch_body.arm as arm
import stretch_body.lift as lift
import stretch_body.pimu as pimu
import stretch_body.wacc as wacc
p=pimu.Pimu()
p.startup()
w=wacc.Wacc()
w.startup()
b=base.Base()
b.startup(threaded=False)
a=arm.Arm()
a.startup(threaded=False)
l=lift.Lift()
l.startup(threaded=False)
print('Version: stretch_body: %s'%stretch_body.__version__)
print('---------- Boards ----------')
print('Pimu:            %s'%p.board_info['board_variant'])
print('Wacc:            %s'%w.board_info['board_variant'])
print('Left wheel:      %s' %b.left_wheel.board_info['board_variant'])
print('Right wheel:     %s' %b.right_wheel.board_info['board_variant'])
print('Lift stepper:    %s' %l.motor.board_info['board_variant'])
print('Arm stepper:     %s' %a.motor.board_info['board_variant'])
print('---------- Firmware ----------')
print('Pimu:            %s'%p.board_info['firmware_version'])
print('Wacc:            %s'%w.board_info['firmware_version'])
print('Left wheel:      %s'%b.left_wheel.board_info['firmware_version'])
print('Right wheel:     %s'%b.right_wheel.board_info['firmware_version'])
print('Lift:            %s'%l.motor.board_info['firmware_version'])
print('Arm:             %s'%a.motor.board_info['firmware_version'])
p.stop()
w.stop()
b.stop()
l.stop()
a.stop()
print("")


subprocess.call(['echo', 'USB Root Hubs'])
subprocess.call(['echo', "-------------------------------------------------------------------------"])
subprocess.call(['lsusb', '-s' ':1'])
subprocess.call(['echo'," "])


subprocess.call(['echo', "USB Bus 1"])
subprocess.call(['echo', "-------------------------------------------------------------------------"])
subprocess.call(['lsusb', '-s','1:'])
subprocess.call(['echo'," "])

subprocess.call(['echo', "USB Bus 2"])
subprocess.call(['echo', "-------------------------------------------------------------------------"])
subprocess.call(['lsusb','-s', '2:'])
subprocess.call(['echo'," "])

subprocess.call(['echo', "USB Bus 3"])
subprocess.call(['echo', "-------------------------------------------------------------------------"])
subprocess.call(['lsusb', '-s', '3:'])
subprocess.call(['echo'," "])

subprocess.call(['echo', "USB Bus 4"])
subprocess.call(['echo', "-------------------------------------------------------------------------"])
subprocess.call(['lsusb', '-s', '4:'])
subprocess.call(['echo'," "])


subprocess.call(['echo', "Hello Robot Devices"])
subprocess.call(['echo', "-------------------------------------------------------------------------"])
#subprocess.call(['ls', '-l', '/dev/hello*'])
os.system('ls -l /dev/hello*')
subprocess.call(['echo'," "])

subprocess.call(['echo', " Hard Drive"])
subprocess.call(['echo', "-------------------------------------------------------------------------"])
subprocess.call(['df','-h', '--type=ext4'])
subprocess.call(['echo',"---------"])
subprocess.call(['sudo','nvme','list'])
subprocess.call(['echo',"---------"])
subprocess.call(['sudo', 'nvme', 'smart-log', '/dev/nvme0n1'])
subprocess.call(['echo',"---------"])
subprocess.call(['echo'," "])
subprocess.call(['echo'," "])

subprocess.call(['echo', "Network"])
subprocess.call(['echo', "-------------------------------------------------------------------------"])
subprocess.call(['ifconfig'])
subprocess.call(['echo'," "])

subprocess.call(['echo', "D435i Camera"])
subprocess.call(['echo', "-------------------------------------------------------------------------"])
subprocess.call(['lsusb','-d', '8086:0b3a'])
subprocess.call(['echo'," "])

subprocess.call(['echo', "Temperatures"])
subprocess.call(['echo', "-------------------------------------------------------------------------"])
subprocess.call(['sensors'])
subprocess.call(['echo'," "])

subprocess.call(['echo', "Reboot History"])
subprocess.call(['echo', "-------------------------------------------------------------------------"])
subprocess.call(['last','reboot'])
subprocess.call(['echo'," "])




#!/usr/bin/env python
import argparse
import math
import yaml
import pathlib
import urdfpy
import pyrender
import warnings

from hello_helpers.gripper_conversion import GripperConversion
import stretch_body.robot
import stretch_body.hello_utils as hu
hu.print_stretch_re_use()
warnings.filterwarnings("ignore")

class URDFVisualizer:
    """The `show` method in this class is modified from the
    original implementation of `urdfpy.URDF.show`. This class
    exists temporarily while the PR for this modification is
    in review.
    """

    def __init__(self, urdf):
        self.urdf = urdf
        self.nodes = None
        self.scene = None
        self.viewer = None

    def show(self, cfg=None, use_collision=False):
        """Visualize the URDF in a given configuration.
        Parameters
        ----------
        cfg : dict or (n), float
            A map from joints or joint names to configuration values for
            each joint, or a list containing a value for each actuated joint
            in sorted order from the base link.
            If not specified, all joints are assumed to be in their default
            configurations.
        use_collision : bool
            If True, the collision geometry is visualized instead of
            the visual geometry.
        """
        if use_collision:
            fk = self.urdf.collision_trimesh_fk(cfg=cfg)
        else:
            fk = self.urdf.visual_trimesh_fk(cfg=cfg)

        self.scene = pyrender.Scene()
        self.nodes = []
        for tm in fk:
            pose = fk[tm]
            mesh = pyrender.Mesh.from_trimesh(tm, smooth=False)
            mesh_node = self.scene.add(mesh, pose=pose)
            self.nodes.append(mesh_node)
        self.viewer = pyrender.Viewer(self.scene, run_in_thread=True, use_raymond_lighting=True)

    def update_pose(self, cfg=None, use_collision=False):
        if use_collision:
            fk = self.urdf.collision_trimesh_fk(cfg=cfg)
        else:
            fk = self.urdf.visual_trimesh_fk(cfg=cfg)

        self.viewer.render_lock.acquire()
        for i, tm in enumerate(fk):
            pose = fk[tm]
            self.scene.set_pose(self.nodes[i], pose=pose)
        self.viewer.render_lock.release()

class StretchState:

    def __init__(self, robot, controller_parameters):
        self.stretch = robot
        self.controller_parameters = controller_parameters.copy()
        self.gripper_conversion = GripperConversion()

    def get_urdf_configuration(self, backlash_state=None):
        if backlash_state is None:
            # use default backlash state
            backlash_state = {'wrist_extension_retracted': False,
                              'head_pan_looked_left': False}

        stretch_status = self.stretch.get_status()

        # set positions of the telescoping joints
        arm_status = stretch_status['arm']
        if backlash_state['wrist_extension_retracted']:
            arm_backlash_correction = self.controller_parameters['arm_retracted_offset']
        else:
            arm_backlash_correction = 0.0
        arm_m = arm_status['pos'] + arm_backlash_correction
        telescoping_link_m = arm_m / 4.0

        lift_status = stretch_status['lift']
        lift_m = lift_status['pos']

        wrist_yaw_status = stretch_status['end_of_arm']['wrist_yaw']
        wrist_yaw_rad = wrist_yaw_status['pos']

        try:
            gripper_status = stretch_status['end_of_arm']['stretch_gripper']
        except KeyError:
            print('Tool must include Stretch Gripper. Exiting...')
            exit(1)
        _, gripper_finger_rad, _, _ = self.gripper_conversion.status_to_all(gripper_status)

        head_pan_status = stretch_status['head']['head_pan']
        if backlash_state['head_pan_looked_left']:
            pan_backlash_correction = self.controller_parameters['pan_looked_left_offset']
        else:
            pan_backlash_correction = 0.0
        head_pan_rad = head_pan_status['pos'] + self.controller_parameters['pan_angle_offset'] + pan_backlash_correction

        head_tilt_status = stretch_status['head']['head_tilt']
        # ignores tilt backlash state
        raw_head_tilt_rad = head_tilt_status['pos']
        if raw_head_tilt_rad > self.controller_parameters['tilt_angle_backlash_transition']:
            tilt_backlash_correction = self.controller_parameters['tilt_looking_up_offset']
        else:
            tilt_backlash_correction = 0.0
        head_tilt_rad = head_tilt_status['pos'] + self.controller_parameters[
            'tilt_angle_offset'] + tilt_backlash_correction

        configuration = {
            'joint_left_wheel': 0.0,
            'joint_right_wheel': 0.0,
            'joint_lift': lift_m,
            'joint_arm_l0': telescoping_link_m,
            'joint_arm_l1': telescoping_link_m,
            'joint_arm_l2': telescoping_link_m,
            'joint_arm_l3': telescoping_link_m,
            'joint_wrist_yaw': wrist_yaw_rad,
            'joint_gripper_finger_left': gripper_finger_rad,
            'joint_gripper_finger_right': gripper_finger_rad,
            'joint_head_pan': head_pan_rad,
            'joint_head_tilt': head_tilt_rad
        }

        return configuration


if __name__ == "__main__":
    parser = argparse.ArgumentParser(description='Python based URDF visualization')
    parser.add_argument('-t', "--trajectory", help="Visualize predefined trajectory", action="store_true")
    parser.add_argument('-p', "--pose", help="Visualize predefined pose", action="store_true")
    parser.add_argument('-c', "--collision", help="Use collision meshes", action="store_true")
    args = parser.parse_args()

    calibration_dir = pathlib.Path(hu.get_fleet_directory()) / 'exported_urdf'
    urdf_path = calibration_dir / 'stretch.urdf'
    controller_params_path = calibration_dir / 'controller_calibration_head.yaml'
    urdf = urdfpy.URDF.load(str(urdf_path.absolute()))
    viz = URDFVisualizer(urdf)
    with open(str(controller_params_path.absolute()), 'r') as f:
        controller_params = yaml.load(f, Loader=yaml.FullLoader)

    if args.trajectory:
        cfg_trajectory = {
            'joint_left_wheel': [0.0, math.pi],
            'joint_right_wheel': [0.0, math.pi],
            'joint_lift': [0.9, 0.7],
            'joint_arm_l0': [0.0, 0.1],
            'joint_arm_l1': [0.0, 0.1],
            'joint_arm_l2': [0.0, 0.1],
            'joint_arm_l3': [0.0, 0.1],
            'joint_wrist_yaw': [math.pi, 0.0],
            'joint_gripper_finger_left': [0.0, 0.25],
            'joint_gripper_finger_right': [0.0, 0.25],
            'joint_head_pan': [0.0, -(math.pi / 2.0)],
            'joint_head_tilt': [0.5, -0.5]
        }
        urdf.animate(cfg_trajectory=cfg_trajectory, use_collision=args.collision)
    elif args.pose:
        cfg_pose = {
            'joint_left_wheel': 0.0,
            'joint_right_wheel': 0.0,
            'joint_lift': 0.6,
            'joint_arm_l0': 0.1,
            'joint_arm_l1': 0.1,
            'joint_arm_l2': 0.1,
            'joint_arm_l3': 0.1,
            'joint_wrist_yaw': math.pi,
            'joint_gripper_finger_left': 0.0,
            'joint_gripper_finger_right': 0.0,
            'joint_head_pan': -(math.pi / 2.0),
            'joint_head_tilt': -0.5
        }
        urdf.show(cfg=cfg_pose, use_collision=args.collision)
    else:
        r = stretch_body.robot.Robot()
        r.startup()
        if not r.is_calibrated():
            print('Exiting because the robot has not been calibrated')
            exit()

        stretch_state = StretchState(r, controller_params)
        viz.show(cfg=stretch_state.get_urdf_configuration(), use_collision=args.collision)
        while viz.viewer.is_active:
            viz.update_pose(cfg=stretch_state.get_urdf_configuration(), use_collision=args.collision)
        r.stop()



#!/usr/bin/env python
from __future__ import print_function
import stretch_body.lift as lift
import argparse
import stretch_body.hello_utils as hu
hu.print_stretch_re_use()

parser=argparse.ArgumentParser(description='Calibrate the lift position by moving to the upper hardstop')
args=parser.parse_args()

l=lift.Lift()
if not l.startup(threaded=False):
    exit()
l.home()
l.stop()




#!/usr/bin/env python
from __future__ import print_function
import sys, tty, termios
import time
import stretch_body.lift as lift
import argparse
import stretch_body.hello_utils as hu
hu.print_stretch_re_use()

parser=argparse.ArgumentParser(description='Jog the lift motion from the keyboard')
args=parser.parse_args()

small_move_m=.01
large_move_m=0.3



l=lift.Lift()
if not l.startup(threaded=False):
    exit()
l.motor.disable_sync_mode()
l.push_command()

def get_keystroke():

    fd=sys.stdin.fileno()
    old_settings=termios.tcgetattr(fd)
    try:
        tty.setraw(sys.stdin.fileno())
        ch=sys.stdin.read(1)
    finally:
        termios.tcsetattr(fd,termios.TCSADRAIN,old_settings)
    return ch

def menu():
    print('--------------')
    print('m: menu')
    print('u / d : small up down')
    print('U / D : large up down')
    print('f: stiffness float')
    print('s: stiffness soft')
    print('h: stiffness hard')
    print('1: rate slow')
    print('2: rate default')
    print('3: rate fast')
    print('4: rate max')
    print('q: quit')
    print('')
    print('Input?')

rate='default'
req_calibration=False
stiffness=1.0
try:
    menu()
    while True:
        c=get_keystroke()
        l.pull_status()
        l.pretty_print()

        if c == 'f':
            stiffness=0.0
            l.move_by(x_m=0, v_m=l.params['motion'][rate]['vel_m'], a_m=l.params['motion'][rate]['accel_m'],
                      stiffness=stiffness, req_calibration=req_calibration)
        if c == 's':
            stiffness=0.3
            l.move_by(x_m=0, v_m=l.params['motion'][rate]['vel_m'], a_m=l.params['motion'][rate]['accel_m'],
                      stiffness=stiffness, req_calibration=req_calibration)
        if c == 'h':
            stiffness=1.0
            l.move_by(x_m=0, v_m=l.params['motion'][rate]['vel_m'], a_m=l.params['motion'][rate]['accel_m'],
                      stiffness=stiffness, req_calibration=req_calibration)

        if c=='1':
            rate='slow'
        if c == '2':
            rate = 'default'
        if c == '3':
            rate = 'fast'
        if c == '4':
            rate = 'max'
        if c=='m':
            menu()
        if c=="Q" or c=='q':
            break
        if c == 'u':
            l.move_by(x_m= small_move_m, v_m=l.params['motion'][rate]['vel_m'], a_m=l.params['motion'][rate]['accel_m'],stiffness=stiffness, req_calibration=req_calibration)
        if c == 'd':
            l.move_by(x_m=-1*small_move_m, v_m=l.params['motion'][rate]['vel_m'], a_m=l.params['motion'][rate]['accel_m'],stiffness=stiffness, req_calibration=req_calibration)
        if c == 'U':
            l.move_by(x_m=large_move_m, v_m=l.params['motion'][rate]['vel_m'], a_m=l.params['motion'][rate]['accel_m'],stiffness=stiffness, req_calibration=req_calibration)
        if c == 'D':
            l.move_by(x_m=-1*large_move_m, v_m=l.params['motion'][rate]['vel_m'], a_m=l.params['motion'][rate]['accel_m'],stiffness=stiffness, req_calibration=req_calibration)
        l.push_command()
        time.sleep(0.1)
except (KeyboardInterrupt, SystemExit):
    pass
l.stop()



#!/usr/bin/env python3
from __future__ import print_function
import stretch_body.scope
from stretch_body.hello_utils import *

import sys
import time
import argparse

parser=argparse.ArgumentParser(description='Test out splined trajectories on the various joint from a GUI or text menu.')
parser.add_argument("--text", '-t', help="Use text options instead of GUI", action="store_true")
parser.add_argument("--preloaded_traj", '-p', help="Load one of three predefined trajectories", choices=['1', '2', '3'], default='1')

group = parser.add_mutually_exclusive_group(required=True)
group.add_argument("--head_pan", help="Test trajectories on the head_pan joint", action="store_true")
group.add_argument("--head_tilt", help="Test trajectories on the head_tilt joint", action="store_true")
group.add_argument("--wrist_yaw", help="CTest trajectories on the wrist_yaw joint", action="store_true")
group.add_argument("--gripper", help="Test trajectories on the stretch_gripper joint", action="store_true")
group.add_argument("--arm", help="Test trajectories on the arm joint", action="store_true")
group.add_argument("--lift", help="Test trajectories on the lift joint", action="store_true")
group.add_argument("--base_translate", help="Test translational trajectories on diff-drive base", action="store_true")
group.add_argument("--base_rotate", help="Test rotational trajectories on diff-drive base", action="store_true")
group.add_argument("--full_body", help="Test trajectories on all joints at once", action="store_true")
args, _ = parser.parse_known_args()

if get_display() is None:
    print('Display server not available. Using text options.')
    args.text = True

def add_waypoints(times,positions,velocities):
    for waypoint in zip(times, positions, velocities):
        j.trajectory.add(waypoint[0], waypoint[1], waypoint[2])

# ################### WRIST YAW ###################################
if args.wrist_yaw:
    import stretch_body.wrist_yaw
    j = stretch_body.wrist_yaw.WristYaw()
    add_waypoint_cb = add_waypoints
    j.startup(threaded=True)
    j_yrange = (j.ticks_to_world_rad(j.params['range_t'][0]),
                j.ticks_to_world_rad(j.params['range_t'][1]))
    #j_vrange = (-3.0, 3.0)
    j_vrange = (-1 * j.params['motion']['trajectory_max']['vel_r'], j.params['motion']['trajectory_max']['vel_r'])
    key_pos='pos'
    j_title='Wrist Yaw Trajectory'
    j_label="Wrist Yaw Joint Range (rad)"
    j_req_calibration=True
    if args.preloaded_traj == '1':
        vtime = [0.0, 5.0, 10.0]
        vpos = [j.status['pos'], deg_to_rad(-45), deg_to_rad(90.0)]
        vvel = [j.status['vel'], deg_to_rad(0.0), deg_to_rad(10.0)]
    elif args.preloaded_traj == '2':
        vtime = [0.0, 5.0, 10.0, 15.0]
        vpos = [j.status['pos'], deg_to_rad(40.0), deg_to_rad(-30.0), deg_to_rad(90.0)]
        vvel = [j.status['vel'], deg_to_rad(90.0), deg_to_rad(-40.0), deg_to_rad(0.0)]
    elif args.preloaded_traj == '3':
        vtime = [0.0, 30.0, 60.0]
        vpos = [j.status['pos'], deg_to_rad(0.0), deg_to_rad(90.0)]
        vvel = [j.status['vel'], deg_to_rad(0.0), deg_to_rad(10.0)]

# ################### HEAD PAN ###################################
if args.head_pan:
    import stretch_body.dynamixel_hello_XL430
    j = stretch_body.dynamixel_hello_XL430.DynamixelHelloXL430('head_pan')
    add_waypoint_cb = add_waypoints
    j.startup(threaded=True)
    j_yrange = (j.ticks_to_world_rad(j.params['range_t'][0]),
                j.ticks_to_world_rad(j.params['range_t'][1]))
    #j_vrange = (-3.0, 3.0)
    j_vrange = (-1 * j.params['motion']['trajectory_max']['vel_r'], j.params['motion']['trajectory_max']['vel_r'])
    key_pos='pos'
    j_title='Head Pan Trajectory'
    j_label="Head Pan Joint Range (rad)"
    j_req_calibration = False

    if args.preloaded_traj == '1':
        vtime = [0.0, 4.0, 8.0, 10.0]
        vpos = [j.status['pos'], deg_to_rad(-180.0), deg_to_rad(60.0), deg_to_rad(0.0)]
        vvel = [j.status['vel'], deg_to_rad(0.0), deg_to_rad(0.0), deg_to_rad(0.0)]
    elif args.preloaded_traj == '2':
        vtime = [0.0, 3.0, 6.0]
        vpos = [j.status['pos'], deg_to_rad(-100.0), deg_to_rad(0.0)]
        vvel = [j.status['vel'], deg_to_rad(0.0), deg_to_rad(0.0)]
    elif args.preloaded_traj == '3':
        vtime = [0.0, 30.0, 60.0]
        vpos = [j.status['pos'], deg_to_rad(-180.0), deg_to_rad(0.0)]
        vvel = [j.status['vel'], deg_to_rad(0.0), deg_to_rad(10.0)]
# ################### GRIPPER ###################################
if args.gripper:
    import stretch_body.stretch_gripper
    j = stretch_body.stretch_gripper.StretchGripper()
    add_waypoint_cb = add_waypoints
    j.startup(threaded=True)
    j_yrange = (j.pct_to_world_rad(j.poses['close']),j.pct_to_world_rad(j.poses['open']))
    #j_vrange = (-3.0, 3.0)
    j_vrange = (-1 * j.params['motion']['trajectory_max']['vel_r'], j.params['motion']['trajectory_max']['vel_r'])
    key_pos='pos'
    j_title='Stretch Gripper Trajectory'
    j_label="Stretch Gripper Joint Range (Rad)"
    j_req_calibration = True

    if args.preloaded_traj == '1':
        vtime = [0.0, 4.0, 8.0, 10.0]
        vpos = [j.status['pos'], j.pct_to_world_rad(-80), j.pct_to_world_rad(40), j.pct_to_world_rad(0)]
        vvel = [j.status['vel'], deg_to_rad(0.0), deg_to_rad(0.0), deg_to_rad(0.0)]
    elif args.preloaded_traj == '2':
        vtime = [0.0, 3.0, 6.0]
        vpos = [j.status['pos'], j.pct_to_world_rad(50.0), j.pct_to_world_rad(-30)]
        vvel = [j.status['vel'], deg_to_rad(0.0), deg_to_rad(0.0), deg_to_rad(0.0)]
    elif args.preloaded_traj == '3':
        vtime = [0.0, 30.0, 60.0]
        vpos = [j.status['pos'], j.pct_to_world_rad(30.0), j.pct_to_world_rad(30)]
        vvel = [j.status['vel'], deg_to_rad(0.0), deg_to_rad(10.0)]

# ################### HEAD TILT ###################################
if args.head_tilt:
    import stretch_body.dynamixel_hello_XL430
    j = stretch_body.dynamixel_hello_XL430.DynamixelHelloXL430('head_tilt')
    add_waypoint_cb = add_waypoints
    j.startup(threaded=True)
    j_yrange = (j.ticks_to_world_rad(j.params['range_t'][0]),
                j.ticks_to_world_rad(j.params['range_t'][1]))
    j_vrange = (-1 * j.params['motion']['trajectory_max']['vel_r'], j.params['motion']['trajectory_max']['vel_r'])
    #j_vrange = (-3.0, 3.0)
    key_pos='pos'
    j_title='Head Tilt Trajectory'
    j_label="Head Tilt Joint Range (rad)"
    j_req_calibration = False

    if args.preloaded_traj == '1':
        vtime = [0.0, 3.0, 7.0, 10.0]
        vpos = [j.status['pos'], deg_to_rad(-90.0), deg_to_rad(20.0), deg_to_rad(0.0)]
        vvel = [j.status['vel'], deg_to_rad(0.0), deg_to_rad(0.0), deg_to_rad(0.0)]
    elif args.preloaded_traj == '2':
        vtime = [0.0, 3.0, 6.0]
        vpos = [j.status['pos'], deg_to_rad(-50.0), deg_to_rad(0.0)]
        vvel = [j.status['vel'], deg_to_rad(0.0), deg_to_rad(0.0)]
    elif args.preloaded_traj == '3':
        vtime = [0.0, 30.0, 60.0]
        vpos = [j.status['pos'], deg_to_rad(-90.0), deg_to_rad(0.0)]
        vvel = [j.status['vel'], deg_to_rad(0.0), deg_to_rad(10.0)]

# ################### ARM ###################################
if args.arm:
    import stretch_body.arm
    j = stretch_body.arm.Arm()
    add_waypoint_cb = add_waypoints
    j.startup(threaded=True)
    j.motor.disable_sync_mode()
    #j.motor.disable_guarded_mode()
    j.push_command()
    j_yrange = (j.params['range_m'][0], j.params['range_m'][1])
    j_vrange = (-1 * j.params['motion']['trajectory_max']['vel_m'],j.params['motion']['trajectory_max']['vel_m'])
    key_pos='pos'
    j_title='Stretch Arm Trajectory'
    j_label="Stretch Arm Joint Range (Rad)"
    j_req_calibration = True

    if args.preloaded_traj == '1':
        vtime = [0.0, 10.0, 20.0]
        vpos = [j.status['pos'], 0.45, 0.0]
        vvel = [j.status['vel'], 0.0, 0.0]
    elif args.preloaded_traj == '2':
        vtime = [0.0, 3.0, 6.0, 9.0]
        vpos = [j.status['pos'], 0.10, 0.3, 0.15]
        vvel = [j.status['vel'], 0.0, 0.0, 0.0]
    elif args.preloaded_traj == '3':
        vtime = [0.0, 30.0, 60.0]
        vpos = [j.status['pos'], 0.45, 0.0]
        vvel = [j.status['vel'], 0.0, -0.06]
# ################### Lift ###################################
if args.lift:
    import stretch_body.lift
    j = stretch_body.lift.Lift()
    add_waypoint_cb = add_waypoints
    j.startup(threaded=True)
    j.motor.disable_sync_mode()
    #j.motor.disable_guarded_mode()
    j.push_command()
    j_yrange = (j.params['range_m'][0], j.params['range_m'][1])
    j_vrange = (-1 * j.params['motion']['trajectory_max']['vel_m'],j.params['motion']['trajectory_max']['vel_m'])
    key_pos='pos'
    j_title='Stretch Lift Trajectory'
    j_label="Stretch Lift Joint Range (Rad)"
    j_req_calibration = True

    if args.preloaded_traj == '1':
        vtime = [0.0, 15.0, 30.0]
        vpos = [j.status['pos'], 0.9, 0.2]
        vvel = [j.status['vel'], 0.0, 0.0]
    elif args.preloaded_traj == '2':
        vtime = [0.0, 3.0, 6.0, 9.0]
        vpos = [j.status['pos'], 0.3, 0.4, 0.2]
        vvel = [j.status['vel'], 0.0, 0.0, 0.0]
    elif args.preloaded_traj == '3':
        vtime = [0.0, 30.0, 60.0]
        vpos = [j.status['pos'], 0.9, 0.2]
        vvel = [j.status['vel'], 0.0, -0.08]

# ################### Base ###################################
if args.base_translate or args.base_rotate:
    import stretch_body.base
    j = stretch_body.base.Base()
    j.startup(threaded=True)
    j_req_calibration = False
    j.first_step = True #Reset odometry
    j.pull_status()
    j.right_wheel.disable_sync_mode()
    j.left_wheel.disable_sync_mode()
    j.push_command()
    if args.base_translate:
        j_yrange = (-0.5, 0.5) #meters
        j_vrange = (-1 * j.params['motion']['trajectory_max']['vel_r'],j.params['motion']['trajectory_max']['accel_r'])
        j_title = 'Stretch Base Translate Trajectory'
        j_label = "Stretch Base Translate Range (Rad)"
        def add_translate_waypoints(times, positions, velocities):
            for waypoint in zip(times, positions, velocities):
                j.trajectory.add(time=waypoint[0], x=waypoint[1], y=0, theta=0, translational_vel=waypoint[2],rotational_vel=0)
        add_waypoint_cb = add_translate_waypoints
        key_pos = 'x'
        if args.preloaded_traj == '1':
            vtime = [0.0, 3.0, 6.0]
            vpos = [j.status['x'], 0.1, 0.0]
            vvel=  [j.status['x_vel'], 0.0, 0.0]
        if args.preloaded_traj == '2':
            vtime = [0.0, 3.0, 6.0,9.0,12.0]
            vpos = [j.status['x'], 0.1, 0.15,0.2,0.0]
            vvel=  [j.status['x_vel'], 0.2, -0.1,-0.05,0.0]
        if args.preloaded_traj == '3':
            vtime = [0.0, 10.0, 15.0, 20.0]
            vpos = [j.status['x'], 0.1, 0.4,0.0]
            vvel=  [j.status['x_vel'], 0.2, -0.1,0.0]

    if args.base_rotate:
        j_yrange = (deg_to_rad(-180),deg_to_rad(180))  # radians
        j_vrange = (-1 * j.params['motion']['trajectory_max']['vel_r'], j.params['motion']['trajectory_max']['accel_r'])
        j_title = 'Stretch Base Rotate Trajectory'
        j_label = "Stretch Base Rotate Range (Rad)"


        def add_rotate_waypoints(times, positions, velocities):
            for waypoint in zip(times, positions, velocities):
                j.trajectory.add(time=waypoint[0], x=0, y=0, theta=waypoint[1], translational_vel=0,rotational_vel=waypoint[2])

        add_waypoint_cb = add_rotate_waypoints
        key_pos = 'theta'
        if args.preloaded_traj == '1':
            vtime = [0.0, 3.0, 6.0]
            vpos = [j.status['theta'], deg_to_rad(90.0), 0.0]
            vvel = [j.status['theta_vel'], 0.0, 0.0]
        if args.preloaded_traj == '2':
            vtime = [0.0, 3.0, 6.0, 9.0, 12.0]
            vpos = [j.status['theta'], deg_to_rad(90.0),deg_to_rad(-45.0),deg_to_rad(45.0), 0.0]
            vvel = [j.status['theta_vel'], deg_to_rad(10.0),deg_to_rad(-5.0),deg_to_rad(-10.0), 0.0]
        if args.preloaded_traj == '3':
            vtime = [0.0, 10.0, 15.0, 20.0]
            vpos = [j.status['theta'], deg_to_rad(45.0),deg_to_rad(-10.0), 0.0]
            vvel = [j.status['theta_vel'], deg_to_rad(5.0),deg_to_rad(-5.0), 0.0]


# ################### Full Body ###################################
if args.full_body:
    import stretch_body.robot

    r = stretch_body.robot.Robot()
    r.startup()

    print('move all joints to initial positions')
    r.arm.move_to(0.0)
    r.lift.move_to(0.2)
    r.push_command()
    r.head.pose('ahead')
    r.end_of_arm.motors['wrist_yaw'].pose('side')
    time.sleep(4.0)

    r.lift.trajectory.add(t_s=0.0, x_m=0.2, v_m=0.0)
    r.lift.trajectory.add(t_s=10.0, x_m=0.9, v_m=0.0)
    r.lift.trajectory.add(t_s=20.0, x_m=0.2, v_m=0.0)
    r.arm.trajectory.add(t_s=0.0, x_m=0.0, v_m=0.0)
    r.arm.trajectory.add(t_s=10.0, x_m=0.5, v_m=0.0)
    r.arm.trajectory.add(t_s=20.0, x_m=0.0, v_m=0.0)

    r.base.trajectory.add(time=0.0, x=0,y=0, theta=deg_to_rad(0.0), translational_vel=0.0, rotational_vel=0.0 )
    r.base.trajectory.add(time=10.0, x=0,y=0, theta=deg_to_rad(180.0), translational_vel=0.0, rotational_vel=0.0)
    r.base.trajectory.add(time=20.0,  x=0,y=0,theta=deg_to_rad(0.0), translational_vel=0.0, rotational_vel=0.0)


    r.end_of_arm.motors['wrist_yaw'].trajectory.add(t_s=0.0, x_r=deg_to_rad(90.0), v_r=0.0)
    r.end_of_arm.motors['wrist_yaw'].trajectory.add(t_s=10.0, x_r=deg_to_rad(-45.0), v_r=0.0)
    r.end_of_arm.motors['wrist_yaw'].trajectory.add(t_s=20.0, x_r=deg_to_rad(90.0), v_r=0.0)
    #r.end_of_arm.motors['wrist_yaw'].trajectory.add_waypoint(t_s=40.0, x_r=deg_to_rad(180.0), v_r=0.0)

    r.head.get_joint('head_tilt').trajectory.add(t_s=0.0, x_r=deg_to_rad(0.0), v_r=0.0)
    r.head.get_joint('head_tilt').trajectory.add(t_s=10.0, x_r=deg_to_rad(-90.0), v_r=0.0)
    r.head.get_joint('head_tilt').trajectory.add(t_s=20.0, x_r=deg_to_rad(0.0), v_r=0.0)

    r.head.get_joint('head_pan').trajectory.add(t_s=0.0, x_r=deg_to_rad(0.0), v_r=0.0)
    r.head.get_joint('head_pan').trajectory.add(t_s=10.0, x_r=deg_to_rad(-90.0), v_r=0.0)
    r.head.get_joint('head_pan').trajectory.add(t_s=20.0, x_r=deg_to_rad(0.0), v_r=0.0)

    print('start follow_trajectory')
    r.follow_trajectory()
    ts = time.time()
    while r.is_trajectory_active():
        #dt = (time.time() - ts) / 20.0  # 0-1
        print('Time remaining: %f'%(20.0-(time.time()-ts)))
        # r.base.set_translate_velocity(v_m=dt*0.5) #Uncomment this for base motion (put up on blocks first!)
        #r.push_command()
        time.sleep(0.1)

    r.stop_trajectory()
    r.stop()
    print('done')
    exit(0)

# ##########################################################################

if not args.text:

    def start_trajectory(times, positions, velocities):
        add_waypoint_cb(times,positions,velocities)
        if not args.base_translate and not args.base_rotate:
            j.follow_trajectory(req_calibration=j_req_calibration, move_to_start_point=False)
        else:
            j.follow_trajectory()

    def sense_trajectory():
        #j.pretty_print()
        if j.is_trajectory_active():
            if args.base_rotate:
                if j.status['theta']>deg_to_rad(180.0): #Handle rollover in odometry
                    return (j.get_trajectory_ts(), j.status['theta']-deg_to_rad(360.0))
            return (j.get_trajectory_ts(), j.status[key_pos])

    def update_trajectory(times, positions, velocities):
        j.trajectory.clear()
        for waypoint in zip(times, positions, velocities):
            j.trajectory.add(waypoint[0], waypoint[1], waypoint[2])

    def stop_trajectory():
        j.stop_trajectory()
        j.trajectory.clear()
        time.sleep(0.25)
        return j.status[key_pos]

    s = stretch_body.scope.TrajectoryScope(vtime, vpos, vvel,
            yrange=j_yrange, vrange=j_vrange, sense_frequency=15,
            title=j_title,
            ylabel=j_label)
    s.start(start_trajectory, sense_trajectory, update_trajectory, stop_trajectory)
else:

    def menu_top():
        print('------ MENU -------')
        print('m: menu')
        print('a: start trajectory')
        print('s: stop trajectory')
        print('q: quit')
        print('-------------------')

    def step_interaction():
        menu_top()
        x=sys.stdin.readline()
        if len(x)>1:
            if x[0]=='m':
                pass
            if x[0]=='a':
                add_waypoint_cb(vtime,vpos,vvel)
                if not args.base_translate and not args.base_rotate:
                    j.follow_trajectory(req_calibration=j_req_calibration, move_to_start_point=True)
                else:
                    j.follow_trajectory()
                print("\nExecuting trajectory: {0}\n".format(j.trajectory))
            if x[0]=='s':
                j.stop_trajectory()
            if x[0]=='q':
                j.stop_trajectory()
                j.stop()
                exit()
        else:
            if j.is_trajectory_active():
                print("\nDuration Remaining: {0}.\nCurrent pos {1}\n".format(j.get_trajectory_time_remaining(), key_pos))
            else:
                print("\nTrajectory inactive\n")

    try:
        while True:
            try:
                step_interaction()
            except (ValueError):
                print('Bad input...')
    except (ThreadServiceExit, KeyboardInterrupt):
        j.stop_trajectory()
        j.stop()



#!/usr/bin/env python
from __future__ import print_function
import stretch_body.robot_params
stretch_body.robot_params.RobotParams.set_logging_level("DEBUG")
import stretch_body.head as head
from stretch_body.hello_utils import *
import sys
import argparse
print_stretch_re_use()

parser=argparse.ArgumentParser(description='Jog the head from the keyboard')
args=parser.parse_args()

h=head.Head()
if not h.startup(threaded=False):
    exit()

v_des=[h.motors['head_pan'].params['motion']['default']['vel'], h.motors['head_tilt'].params['motion']['default']['vel']]
a_des=[h.motors['head_pan'].params['motion']['default']['accel'], h.motors['head_tilt'].params['motion']['default']['accel']]

def menu_top():
    print('------ MENU -------')
    print('m: menu')
    print('a: increment pan 10 deg')
    print('b: decrement pan 10 deg')
    print('c: increment tilt 10 deg')
    print('d: decrement tilt 10 deg')
    print('e: ahead')
    print('f: back')
    print('g: tool')
    print('h: wheels')
    print('i: left')
    print('j: up')
    print('p: pan go to pos ticks')
    print('t: tilt go to pos ticks')
    print('x: home')
    print('1: speed slow')
    print('2: speed default')
    print('3: speed fast')
    print('4: speed max')
    print('-------------------')

def step_interaction():
    global v_des, a_des
    menu_top()
    x=sys.stdin.readline()
    if len(x)>1:
        if x[0]=='m':
            menu_top()
        if x[0]=='p':
            p = float(x[1:])
            p = h.motors['head_pan'].ticks_to_world_rad(p)
            h.move_to('head_pan', p, v_des[0], a_des[0])

        if x[0]=='x':
            h.home()

        if x[0]=='t':
            t = float(x[1:])
            t= h.motors['head_tilt'].ticks_to_world_rad(t)
            h.move_to('head_tilt', t, v_des[1], a_des[1])

        if x[0] == 'a':
            h.move_by('head_pan',deg_to_rad(10), v_des[0], a_des[0])
        if x[0] == 'b':
            h.move_by('head_pan', deg_to_rad(-10), v_des[0], a_des[0])
        if x[0] == 'c':
            h.move_by('head_tilt', deg_to_rad(10), v_des[1], a_des[1])
        if x[0] == 'd':
            h.move_by('head_tilt', deg_to_rad(-10), v_des[1], a_des[1])

        if x[0] == '1':
            v_des = [h.motors['head_pan'].params['motion']['slow']['vel'], h.motors['head_tilt'].params['motion']['slow']['vel']]
            a_des = [h.motors['head_pan'].params['motion']['slow']['accel'], h.motors['head_tilt'].params['motion']['slow']['accel']]

        if x[0] == '2':
            v_des = [h.motors['head_pan'].params['motion']['default']['vel'], h.motors['head_tilt'].params['motion']['default']['vel']]
            a_des = [h.motors['head_pan'].params['motion']['default']['accel'], h.motors['head_tilt'].params['motion']['default']['accel']]

        if x[0] == '3':
            v_des = [h.motors['head_pan'].params['motion']['fast']['vel'], h.motors['head_tilt'].params['motion']['fast']['vel']]
            a_des = [h.motors['head_pan'].params['motion']['fast']['accel'], h.motors['head_tilt'].params['motion']['fast']['accel']]

        if x[0] == '4':
            v_des = [h.motors['head_pan'].params['motion']['max']['vel'], h.motors['head_tilt'].params['motion']['max']['vel']]
            a_des = [h.motors['head_pan'].params['motion']['max']['accel'], h.motors['head_tilt'].params['motion']['max']['accel']]

        if x[0] == 'e':
            h.pose('ahead', v_des, a_des)
        if x[0] == 'f':
            h.pose('back', v_des, a_des)
        if x[0] == 'g':
            h.pose('tool', v_des, a_des)
        if x[0] == 'h':
            h.pose('wheels', v_des, a_des)
        if x[0] == 'i':
            h.pose('left', v_des, a_des)
        if x[0] == 'j':
            h.pose('up', v_des, a_des)
    else:
        h.pretty_print()



try:
    while True:
        try:
            step_interaction()
        except (ValueError):
            print('Bad input...')
        h.pull_status()
except (ThreadServiceExit, KeyboardInterrupt):
    h.stop()




#!/usr/bin/env python
from __future__ import print_function
import time
import stretch_body.wacc as wacc
from stretch_body.hello_utils import *
import argparse
print_stretch_re_use()
# ###################################


parser=argparse.ArgumentParser(description='Visualize Wacc (Wrist+Accel) board data with an oscilloscope')
parser.add_argument("--ax", help="Scope  accelerometer AX",action="store_true")
parser.add_argument("--ay", help="Scope  accelerometer AY",action="store_true")
parser.add_argument("--az", help="Scope  accelerometer AZ",action="store_true")
parser.add_argument("--a0", help="Scope  analog-in-0",action="store_true")
parser.add_argument("--d0", help="Scope  digital-in-0",action="store_true")
parser.add_argument("--d1", help="Scope  digital-in-1",action="store_true")
parser.add_argument("--tap", help="Scope single tap",action="store_true")

args = parser.parse_args()


p=wacc.Wacc()
if not p.startup():
    exit()


#Accel
if args.ax:
    import stretch_body.scope as scope
    s = scope.Scope(yrange=[8,11], title='AX')
    try:
        while True:
            p.pull_status()
            time.sleep(0.02)
            print('AX',p.status['ax'])
            s.step_display(p.status['ax'])
    except (SystemExit, KeyboardInterrupt):
        p.stop()

if args.ay:
    import stretch_body.scope as scope
    s = scope.Scope(yrange=[-2,2], title='AY')
    try:
        while True:
            p.pull_status()
            time.sleep(0.02)
            print('AY',p.status['ay'])
            s.step_display(p.status['ay'])
    except (SystemExit, KeyboardInterrupt):
        p.stop()

if args.az:
    import stretch_body.scope as scope
    s = scope.Scope(yrange=[-2,2], title='AZ')
    try:
        while True:
            p.pull_status()
            time.sleep(0.02)
            print('AZ',p.status['az'])
            s.step_display(p.status['az'])
    except (SystemExit, KeyboardInterrupt):
        p.stop()



if args.a0:
    import stretch_body.scope as scope
    s = scope.Scope( title='A0')
    try:
        while True:
            p.pull_status()
            time.sleep(0.02)
            print('A0',p.status['a0'])
            s.step_display(p.status['a0'])
    except (SystemExit, KeyboardInterrupt):
        p.stop()

if args.d0:
    import stretch_body.scope as scope
    s = scope.Scope( title='D0')
    try:
        while True:
            p.pull_status()
            time.sleep(0.02)
            print('D0',p.status['d0'])
            s.step_display(p.status['d0'])
    except (SystemExit, KeyboardInterrupt):
        p.stop()

if args.d1:
    import stretch_body.scope as scope
    s = scope.Scope( title='D1')
    try:
        while True:
            p.pull_status()
            time.sleep(0.02)
            print('D1',p.status['d1'])
            s.step_display(p.status['d1'])
    except (SystemExit, KeyboardInterrupt):
        p.stop()


import unittest
import stretch_body.robot_params
stretch_body.robot_params.RobotParams.set_logging_level("DEBUG")

# import subprocess
# import sys

import subprocess, threading

import shlex
from subprocess import Popen, PIPE
from threading import Timer

def run(cmd, timeout_sec):
    proc = Popen(shlex.split(cmd), stdout=PIPE, stderr=PIPE)
    timer = Timer(timeout_sec, proc.kill)
    try:
        timer.start()
        stdout, stderr = proc.communicate()
        print(stderr)
    finally:
        print('\r\n')
        timer.cancel()
        return proc.returncode

tools=['stretch_about.py','stretch_arm_home.py -h','stretch_arm_jog.py','stretch_audio_test.py',
        'stretch_base_jog.py','stretch_gripper_home.py -h', 'stretch_gripper_jog.py','stretch_hardware_echo.py',
        'stretch_head_jog.py','stretch_lift_home.py -h','stretch_lift_jog.py', 'stretch_params.py','stretch_pimu_jog.py',
        'stretch_pimu_scope.py --ax','stretch_respeaker_test.py', 'stretch_robot_battery_check.py','stretch_robot_dynamixel_reboot.py',
        'stretch_robot_home.py -h','stretch_robot_jog.py','stretch_robot_keyboard_teleop.py','stretch_robot_monitor.py',
        'stretch_robot_system_check.py','stretch_rp_lidar_jog.py --range',
        'stretch_wacc_jog.py','stretch_wacc_scope.py','stretch_wrist_yaw_jog.py','stretch_xbox_controller_teleop.py']

tool_py3_only=['stretch_robot_urdf_visualizer.py']

#This has an issue where stdout / console gets in a state
#Where print statement dont' new line. To fix...
class TestToolsLaunch(unittest.TestCase):

    def test_tools_launch_py2(self):
        path='../bin/'
        for t in tools:
            print('Py2: Launching %s ...'%t)
            r=run('python2 '+path+t,timeout_sec=2.0)
            self.assertNotEqual(r,1)

    def test_tools_launch_py3(self):
        path='../bin/'
        for t in tools:
            print('Py3: Launching %s ...'%t)
            r=run('python3 '+path+t,timeout_sec=2.0)
            self.assertNotEqual(r,1)
        for t in tool_py3_only:
            print('Py3: Launching %s ...' % t)
            r = run('python3 ' + path + t, timeout_sec=2.0)
            self.assertNotEqual(r, 1)


#!/usr/bin/env python
from __future__ import print_function
import sys, tty, termios
import time
import stretch_body.arm as arm
import argparse
import stretch_body.hello_utils as hu
hu.print_stretch_re_use()

parser=argparse.ArgumentParser(description='Jog the arm motion from the keyboard')
args=parser.parse_args()

small_move_m=.01
large_move_m=0.1

a=arm.Arm()
if not a.startup(threaded=False):
    exit()
a.motor.disable_sync_mode()
a.push_command()

def get_keystroke():

    fd=sys.stdin.fileno()
    old_settings=termios.tcgetattr(fd)
    try:
        tty.setraw(sys.stdin.fileno())
        ch=sys.stdin.read(1)
    finally:
        termios.tcsetattr(fd,termios.TCSADRAIN,old_settings)
    return ch

def menu():
    print('--------------')
    print('m: menu')
    print('i / o : small in out')
    print('I / O : large in out')
    print('f: stiffness float')
    print('s: stiffness soft')
    print('h: stiffness hard')
    print('1: rate slow')
    print('2: rate default')
    print('3: rate fast')
    print('4: rate max')
    print('q: quit')
    print('')
    print('Input?')

rate='default'
req_calibration=False
stiffness=1.0
try:

    while True:
        menu()
        c=get_keystroke()
        a.pull_status()
        a.pretty_print()

        if c=='1':
            rate='slow'
        if c == '2':
            rate = 'default'
        if c == '3':
            rate = 'fast'
        if c == '4':
            rate = 'max'
        if c == 'f':
            stiffness=0.0
            a.move_by(x_m=-0, v_m=a.params['motion'][rate]['vel_m'],
                      a_m=a.params['motion'][rate]['accel_m'], stiffness=stiffness, req_calibration=req_calibration)
        if c == 's':
            stiffness=0.3
            a.move_by(x_m=0, v_m=a.params['motion'][rate]['vel_m'],
                      a_m=a.params['motion'][rate]['accel_m'], stiffness=stiffness, req_calibration=req_calibration)
        if c == 'h':
            stiffness=1.0
            a.move_by(x_m=0, v_m=a.params['motion'][rate]['vel_m'],
                      a_m=a.params['motion'][rate]['accel_m'], stiffness=stiffness, req_calibration=req_calibration)

        if c=='m':
            menu()
        if c=="Q" or c=='q':
            break
        if c == 'i':
            a.move_by(x_m= -1*small_move_m, v_m=a.params['motion'][rate]['vel_m'], a_m=a.params['motion'][rate]['accel_m'],stiffness=stiffness, req_calibration=req_calibration)
        if c == 'o':
            a.move_by(x_m=small_move_m, v_m=a.params['motion'][rate]['vel_m'], a_m=a.params['motion'][rate]['accel_m'], stiffness=stiffness,req_calibration=req_calibration)
        if c == 'I':
            a.move_by(x_m=-1*large_move_m, v_m=a.params['motion'][rate]['vel_m'], a_m=a.params['motion'][rate]['accel_m'],stiffness=stiffness,req_calibration=req_calibration)
        if c == 'O':
            a.move_by(x_m=large_move_m, v_m=a.params['motion'][rate]['vel_m'], a_m=a.params['motion'][rate]['accel_m'],stiffness=stiffness,req_calibration=req_calibration)
        a.push_command()
        time.sleep(0.1)
except (KeyboardInterrupt, SystemExit):
    pass
a.stop()



#!/usr/bin/env python
from __future__ import print_function
import time
import stretch_body.robot as robot
import os, fnmatch
import subprocess
from colorama import Fore, Back, Style
import argparse
import stretch_body.hello_utils as hu
from stretch_body.dynamixel_XL430 import *
hu.print_stretch_re_use()


parser=argparse.ArgumentParser(description='Check that all robot hardware is present and reporting sane values')
args=parser.parse_args()

# #####################################################
def val_in_range(val_name, val,vmin, vmax):
    p=val <=vmax and val>=vmin
    if p:
        print(Fore.GREEN +'[Pass] ' + val_name + ' = ' + str(val))
    else:
        print(Fore.RED +'[Fail] ' + val_name + ' = ' +str(val)+ ' out of range ' +str(vmin) + ' to ' + str(vmax))

def val_is_not(val_name, val,vnot):
    if val is not vnot:
        print(Fore.GREEN +'[Pass] ' + val_name + ' = ' + str(val))
    else:
        print(Fore.RED +'[Fail] ' + val_name + ' = ' +str(val))

#Turn off logging so get a clean output
import logging
#logging.disable(logging.CRITICAL)
r=robot.Robot()
r.startup()

# #####################################################
print(Style.RESET_ALL)
print('---- Checking Devices ----')
robot_devices={'hello-wacc':0, 'hello-motor-left-wheel':0,'hello-pimu':0, 'hello-lrf':0,'hello-dynamixel-head':0,'hello-dynamixel-wrist':0,'hello-motor-arm':0,'hello-motor-right-wheel':0,
               'hello-motor-lift':0,'hello-respeaker':0}

listOfFiles = os.listdir('/dev')
pattern = "hello*"
for entry in listOfFiles:
    if fnmatch.fnmatch(entry, pattern):
            robot_devices[entry]=1
for k in robot_devices.keys():
    if robot_devices[k]:
        print(Fore.GREEN +'[Pass] : '+k)
    else:
        print(Fore.RED +'[Fail] : '+ k)
# #####################################################

print(Style.RESET_ALL)
if robot_devices['hello-pimu']:
    print('---- Checking Pimu ----')
    p=r.pimu
    val_in_range('Voltage',p.status['voltage'], vmin=p.config['low_voltage_alert'], vmax=14.5)
    val_in_range('Current',p.status['current'], vmin=0.5, vmax=p.config['high_current_alert'])
    val_in_range('Temperature',p.status['temp'], vmin=10, vmax=40)
    val_in_range('Cliff-0',p.status['cliff_range'][0], vmin=p.config['cliff_thresh'], vmax=20)
    val_in_range('Cliff-1',p.status['cliff_range'][1], vmin=p.config['cliff_thresh'], vmax=20)
    val_in_range('Cliff-2',p.status['cliff_range'][2], vmin=p.config['cliff_thresh'], vmax=20)
    val_in_range('Cliff-3',p.status['cliff_range'][3], vmin=p.config['cliff_thresh'], vmax=20)
    val_in_range('IMU AZ',p.status['imu']['az'], vmin=-10.1, vmax=-9.5)
    val_in_range('IMU Pitch', hu.rad_to_deg(p.status['imu']['pitch']), vmin=-12, vmax=12)
    val_in_range('IMU Roll', hu.rad_to_deg(p.status['imu']['roll']), vmin=-12, vmax=12)
    print(Style.RESET_ALL)

# #####################################################
print(Style.RESET_ALL)

if robot_devices['hello-dynamixel-wrist']:
    print('---- Checking EndOfArm ----')
    w = r.end_of_arm
    try:
        for mk in w.motors.keys():
            if w.motors[mk].do_ping():
                print(Fore.GREEN +'[Pass] Ping of: '+mk)
                if w.motors[mk].params['req_calibration']:
                    if w.motors[mk].motor.is_calibrated():
                        print(Fore.GREEN + '[Pass] Calibrated: ' + mk)
                    else:
                        print(Fore.RED + '[Fail] Not Calibrated: ' + mk)
            else:
                print(Fore.RED + '[Fail] Ping of: ' + mk)
            print(Style.RESET_ALL)
    except(IOError, DynamixelCommError):
        print(Fore.RED + '[Fail] Startup of EndOfArm')
# #####################################################
print(Style.RESET_ALL)
if robot_devices['hello-dynamixel-head']:
    print('---- Checking Head ----')
    h = r.head
    try:
        for mk in h.motors.keys():
            if h.motors[mk].do_ping():
                print(Fore.GREEN +'[Pass] Ping of: '+mk)
            else:
                print(Fore.RED + '[Fail] Ping of: ' + mk)
            print(Style.RESET_ALL)
    except(IOError, DynamixelCommError):
        print(Fore.RED + '[Fail] Startup of EndOfArm')
# #####################################################
print(Style.RESET_ALL)
if robot_devices['hello-wacc']:
    print('---- Checking Wacc ----')
    w=r.wacc
    val_in_range('AX',w.status['ax'], vmin=8.0, vmax=11.0)
    print(Style.RESET_ALL)

# #####################################################
print(Style.RESET_ALL)
if robot_devices['hello-motor-left-wheel']:
    print('---- Checking hello-motor-left-wheel ----')
    m = r.base.left_wheel
    val_is_not('Position',m.status['pos'], vnot=0)
    print(Style.RESET_ALL)
    m.stop()
# #####################################################
print(Style.RESET_ALL)
if robot_devices['hello-motor-right-wheel']:
    print('---- Checking hello-motor-right-wheel ----')
    m = r.base.right_wheel
    val_is_not('Position',m.status['pos'], vnot=0)
    print(Style.RESET_ALL)

# #####################################################
print(Style.RESET_ALL)
if robot_devices['hello-motor-arm']:
    print('---- Checking hello-motor-arm ----')
    m = r.arm.motor
    val_is_not('Position',m.status['pos'], vnot=0)
    val_is_not('Position Calibrated', m.status['pos_calibrated'], vnot=False)
    print(Style.RESET_ALL)

# #####################################################
print(Style.RESET_ALL)
if robot_devices['hello-motor-lift']:
    print('---- Checking hello-motor-lift ----')
    m = r.lift.motor
    val_is_not('Position',m.status['pos'], vnot=0)
    val_is_not('Position Calibrated', m.status['pos_calibrated'], vnot=False)
    print(Style.RESET_ALL)

# #####################################################
print(Style.RESET_ALL)
print ('---- Checking for Intel D435i ----')
cmd = "lsusb -d 8086:0b3a"
returned_value = subprocess.call(cmd,shell=True)  # returns the exit code in unix
if returned_value==0:
    print(Fore.GREEN + '[Pass] : Device found ')
else:
    print(Fore.RED + '[Fail] : No device found')
r.stop()


#!/usr/bin/env python
from __future__ import print_function
import pyaudio
import wave
import numpy as np
import usb.core
import struct
import time
import os
import sys
from contextlib import contextmanager
import stretch_body.hello_utils as hu
hu.print_stretch_re_use()


@contextmanager
def ignore_stderr():
    devnull = None
    try:
        devnull = os.open(os.devnull, os.O_WRONLY)
        stderr = os.dup(2)
        sys.stderr.flush()
        os.dup2(devnull, 2)
        try:
            yield
        finally:
            os.dup2(stderr, 2)
            os.close(stderr)
    finally:
        if devnull is not None:
            os.close(devnull)


# parameter list
# name: (id, offset, type, max, min , r/w, info)
PARAMETERS = {
    'AECFREEZEONOFF': (18, 7, 'int', 1, 0, 'rw', 'Adaptive Echo Canceler updates inhibit.', '0 = Adaptation enabled', '1 = Freeze adaptation, filter only'),
    'AECNORM': (18, 19, 'float', 16, 0.25, 'rw', 'Limit on norm of AEC filter coefficients'),
    'AECPATHCHANGE': (18, 25, 'int', 1, 0, 'ro', 'AEC Path Change Detection.', '0 = false (no path change detected)', '1 = true (path change detected)'),
    'RT60': (18, 26, 'float', 0.9, 0.25, 'ro', 'Current RT60 estimate in seconds'),
    'HPFONOFF': (18, 27, 'int', 3, 0, 'rw', 'High-pass Filter on microphone signals.', '0 = OFF', '1 = ON - 70 Hz cut-off', '2 = ON - 125 Hz cut-off', '3 = ON - 180 Hz cut-off'),
    'RT60ONOFF': (18, 28, 'int', 1, 0, 'rw', 'RT60 Estimation for AES. 0 = OFF 1 = ON'),
    'AECSILENCELEVEL': (18, 30, 'float', 1, 1e-09, 'rw', 'Threshold for signal detection in AEC [-inf .. 0] dBov (Default: -80dBov = 10log10(1x10-8))'),
    'AECSILENCEMODE': (18, 31, 'int', 1, 0, 'ro', 'AEC far-end silence detection status. ', '0 = false (signal detected) ', '1 = true (silence detected)'),
    'AGCONOFF': (19, 0, 'int', 1, 0, 'rw', 'Automatic Gain Control. ', '0 = OFF ', '1 = ON'),
    'AGCMAXGAIN': (19, 1, 'float', 1000, 1, 'rw', 'Maximum AGC gain factor. ', '[0 .. 60] dB (default 30dB = 20log10(31.6))'),
    'AGCDESIREDLEVEL': (19, 2, 'float', 0.99, 1e-08, 'rw', 'Target power level of the output signal. ', '[-inf .. 0] dBov (default: -23dBov = 10log10(0.005))'),
    'AGCGAIN': (19, 3, 'float', 1000, 1, 'rw', 'Current AGC gain factor. ', '[0 .. 60] dB (default: 0.0dB = 20log10(1.0))'),
    'AGCTIME': (19, 4, 'float', 1, 0.1, 'rw', 'Ramps-up / down time-constant in seconds.'),
    'CNIONOFF': (19, 5, 'int', 1, 0, 'rw', 'Comfort Noise Insertion.', '0 = OFF', '1 = ON'),
    'FREEZEONOFF': (19, 6, 'int', 1, 0, 'rw', 'Adaptive beamformer updates.', '0 = Adaptation enabled', '1 = Freeze adaptation, filter only'),
    'STATNOISEONOFF': (19, 8, 'int', 1, 0, 'rw', 'Stationary noise suppression.', '0 = OFF', '1 = ON'),
    'GAMMA_NS': (19, 9, 'float', 3, 0, 'rw', 'Over-subtraction factor of stationary noise. min .. max attenuation'),
    'MIN_NS': (19, 10, 'float', 1, 0, 'rw', 'Gain-floor for stationary noise suppression.', '[-inf .. 0] dB (default: -16dB = 20log10(0.15))'),
    'NONSTATNOISEONOFF': (19, 11, 'int', 1, 0, 'rw', 'Non-stationary noise suppression.', '0 = OFF', '1 = ON'),
    'GAMMA_NN': (19, 12, 'float', 3, 0, 'rw', 'Over-subtraction factor of non- stationary noise. min .. max attenuation'),
    'MIN_NN': (19, 13, 'float', 1, 0, 'rw', 'Gain-floor for non-stationary noise suppression.', '[-inf .. 0] dB (default: -10dB = 20log10(0.3))'),
    'ECHOONOFF': (19, 14, 'int', 1, 0, 'rw', 'Echo suppression.', '0 = OFF', '1 = ON'),
    'GAMMA_E': (19, 15, 'float', 3, 0, 'rw', 'Over-subtraction factor of echo (direct and early components). min .. max attenuation'),
    'GAMMA_ETAIL': (19, 16, 'float', 3, 0, 'rw', 'Over-subtraction factor of echo (tail components). min .. max attenuation'),
    'GAMMA_ENL': (19, 17, 'float', 5, 0, 'rw', 'Over-subtraction factor of non-linear echo. min .. max attenuation'),
    'NLATTENONOFF': (19, 18, 'int', 1, 0, 'rw', 'Non-Linear echo attenuation.', '0 = OFF', '1 = ON'),
    'NLAEC_MODE': (19, 20, 'int', 2, 0, 'rw', 'Non-Linear AEC training mode.', '0 = OFF', '1 = ON - phase 1', '2 = ON - phase 2'),
    'SPEECHDETECTED': (19, 22, 'int', 1, 0, 'ro', 'Speech detection status.', '0 = false (no speech detected)', '1 = true (speech detected)'),
    'FSBUPDATED': (19, 23, 'int', 1, 0, 'ro', 'FSB Update Decision.', '0 = false (FSB was not updated)', '1 = true (FSB was updated)'),
    'FSBPATHCHANGE': (19, 24, 'int', 1, 0, 'ro', 'FSB Path Change Detection.', '0 = false (no path change detected)', '1 = true (path change detected)'),
    'TRANSIENTONOFF': (19, 29, 'int', 1, 0, 'rw', 'Transient echo suppression.', '0 = OFF', '1 = ON'),
    'VOICEACTIVITY': (19, 32, 'int', 1, 0, 'ro', 'VAD voice activity status.', '0 = false (no voice activity)', '1 = true (voice activity)'),
    'STATNOISEONOFF_SR': (19, 33, 'int', 1, 0, 'rw', 'Stationary noise suppression for ASR.', '0 = OFF', '1 = ON'),
    'NONSTATNOISEONOFF_SR': (19, 34, 'int', 1, 0, 'rw', 'Non-stationary noise suppression for ASR.', '0 = OFF', '1 = ON'),
    'GAMMA_NS_SR': (19, 35, 'float', 3, 0, 'rw', 'Over-subtraction factor of stationary noise for ASR. ', '[0.0 .. 3.0] (default: 1.0)'),
    'GAMMA_NN_SR': (19, 36, 'float', 3, 0, 'rw', 'Over-subtraction factor of non-stationary noise for ASR. ', '[0.0 .. 3.0] (default: 1.1)'),
    'MIN_NS_SR': (19, 37, 'float', 1, 0, 'rw', 'Gain-floor for stationary noise suppression for ASR.', '[-inf .. 0] dB (default: -16dB = 20log10(0.15))'),
    'MIN_NN_SR': (19, 38, 'float', 1, 0, 'rw', 'Gain-floor for non-stationary noise suppression for ASR.', '[-inf .. 0] dB (default: -10dB = 20log10(0.3))'),
    'GAMMAVAD_SR': (19, 39, 'float', 1000, 0, 'rw', 'Set the threshold for voice activity detection.', '[-inf .. 60] dB (default: 3.5dB 20log10(1.5))'),
    # 'KEYWORDDETECT': (20, 0, 'int', 1, 0, 'ro', 'Keyword detected. Current value so needs polling.'),
    'DOAANGLE': (21, 0, 'int', 359, 0, 'ro', 'DOA angle. Current value. Orientation depends on build configuration.')
}


class Tuning:
    TIMEOUT = 100000

    def __init__(self, dev):
        self.dev = dev

    def write(self, name, value):
        try:
            data = PARAMETERS[name]
        except KeyError:
            return

        if data[5] == 'ro':
            raise ValueError('{} is read-only'.format(name))

        id = data[0]

        # 4 bytes offset, 4 bytes value, 4 bytes type
        if data[2] == 'int':
            payload = struct.pack(b'iii', data[1], int(value), 1)
        else:
            payload = struct.pack(b'ifi', data[1], float(value), 0)

        self.dev.ctrl_transfer(
            usb.util.CTRL_OUT | usb.util.CTRL_TYPE_VENDOR | usb.util.CTRL_RECIPIENT_DEVICE,
            0, 0, id, payload, self.TIMEOUT)

    def read(self, name):
        try:
            data = PARAMETERS[name]
        except KeyError:
            return

        id = data[0]

        cmd = 0x80 | data[1]
        if data[2] == 'int':
            cmd |= 0x40

        length = 8

        response = self.dev.ctrl_transfer(
            usb.util.CTRL_IN | usb.util.CTRL_TYPE_VENDOR | usb.util.CTRL_RECIPIENT_DEVICE,
            0, cmd, id, length, self.TIMEOUT)

        response = struct.unpack(b'ii', response.tostring())

        if data[2] == 'int':
            result = response[0]
        else:
            result = response[0] * (2.**response[1])

        return result

    def set_vad_threshold(self, db):
        self.write('GAMMAVAD_SR', db)

    def is_voice(self):
        return self.read('VOICEACTIVITY')

    @property
    def direction(self):
        return self.read('DOAANGLE')

    @property
    def version(self):
        return self.dev.ctrl_transfer(
            usb.util.CTRL_IN | usb.util.CTRL_TYPE_VENDOR | usb.util.CTRL_RECIPIENT_DEVICE,
            0, 0x80, 0, 1, self.TIMEOUT)[0]

    def close(self):
        """
        close the interface
        """
        usb.util.dispose_resources(self.dev)


def get_respeaker_device_id():
    with ignore_stderr():
        p = pyaudio.PyAudio()
    info = p.get_host_api_info_by_index(0)
    num_devices = info.get('deviceCount')

    device_id = -1
    for i in range(num_devices):
        if (p.get_device_info_by_host_api_device_index(0, i).get('maxInputChannels')) > 0:
            if "ReSpeaker" in p.get_device_info_by_host_api_device_index(0, i).get('name'):
                device_id = i

    return device_id


RESPEAKER_RATE = 16000
RESPEAKER_CHANNELS = 6 # must flash 6_channels_firmware.bin first
RESPEAKER_WIDTH = 2
RESPEAKER_INDEX = get_respeaker_device_id()
CHUNK = 1024


def record_audio(seconds=3):
    p = pyaudio.PyAudio()
    stream = p.open(rate=RESPEAKER_RATE,
                    format=p.get_format_from_width(RESPEAKER_WIDTH),
                    channels=RESPEAKER_CHANNELS,
                    input=True,
                    input_device_index=RESPEAKER_INDEX)

    frames = []
    for i in range(0, int(RESPEAKER_RATE / CHUNK * seconds)):
        data = stream.read(CHUNK)
        a = np.fromstring(data,dtype=np.int16)[0::6] # extracts fused channel 0
        frames.append(a.tostring())

    stream.stop_stream()
    stream.close()
    p.terminate()

    return frames


def save_wav(frames, fname="output.wav"):
    p = pyaudio.PyAudio()
    wf = wave.open(fname, 'wb')
    wf.setnchannels(1)
    wf.setsampwidth(p.get_sample_size(p.get_format_from_width(RESPEAKER_WIDTH)))
    wf.setframerate(RESPEAKER_RATE)
    wf.writeframes(b''.join(frames))
    wf.close()


def play_audio(frames):
    p = pyaudio.PyAudio()
    stream = p.open(rate=RESPEAKER_RATE,
                    format=p.get_format_from_width(RESPEAKER_WIDTH),
                    channels=1,
                    output=True)

    for f in frames:
        stream.write(f, CHUNK)

    stream.stop_stream()
    stream.close()
    p.terminate()

    return frames


if __name__ == "__main__":
    dev = usb.core.find(idVendor=0x2886, idProduct=0x0018)
    try:
        printed_wait_statement = False
        if dev:
            respeaker = Tuning(dev)
            while True:
                try:
                    if not printed_wait_statement and respeaker.is_voice() == 0:
                        print("\n* waiting for audio...")
                        printed_wait_statement = True
                    else:
                        if respeaker.is_voice() == 1:
                            print("* recording 3 seconds")
                            frames = record_audio()
                            print("* done")
                            time.sleep(1)
                            print("* playing audio")
                            play_audio(frames)
                            print("* done")
                            time.sleep(1)
                            printed_wait_statement = False
                        time.sleep(0.01)
                except KeyboardInterrupt:
                    break
    except usb.core.USBError:
        print('Respeaker not on USB bus')



#!/usr/bin/env python
from __future__ import print_function
import stretch_body.robot_params
stretch_body.robot_params.RobotParams.set_logging_level("DEBUG")
from stretch_body.hello_utils import *
import sys
import stretch_body.wrist_yaw as wrist_yaw
import argparse
print_stretch_re_use()

parser=argparse.ArgumentParser(description='Jog the wrist_yaw joint from the keyboard')
args=parser.parse_args()

poses = {'zero':0, 'left':deg_to_rad(90), 'right': deg_to_rad(-70)}
w=wrist_yaw.WristYaw()
if not w.startup(threaded=False):
    exit()

v_des=w.params['motion']['default']['vel']
a_des=w.params['motion']['default']['accel']

def menu_top():
    print('------ MENU -------')
    print('m: menu')
    print('a: increment 15 deg')
    print('b: decrement 15 deg')
    print('p: position (ticks)')
    print('z: zero')
    print('l: left')
    print('r: right')
    print('1: speed slow')
    print('2: speed default')
    print('3: speed fast')
    print('4: speed max')
    print('-------------------')

def step_interaction():
    global v_des, a_des
    menu_top()
    x=sys.stdin.readline()
    if len(x)>1:
        if x[0]=='m':
            menu_top()
        if x[0]=='p':
            p = w.ticks_to_world_rad(float(x[1:]))
            w.move_to(p,v_des,a_des)

        if x[0] == 'a':
            w.move_by(deg_to_rad(15), v_des, a_des)

        if x[0] == 'b':
            w.move_by(deg_to_rad(-15), v_des, a_des)

        if x[0] == '1':
            v_des = w.params['motion']['slow']['vel']
            a_des = w.params['motion']['slow']['accel']

        if x[0] == '2':
            v_des = w.params['motion']['default']['vel']
            a_des = w.params['motion']['default']['accel']

        if x[0] == '3':
            v_des = w.params['motion']['fast']['vel']
            a_des = w.params['motion']['fast']['accel']

        if x[0] == '4':
            v_des = w.params['motion']['max']['vel']
            a_des = w.params['motion']['max']['accel']

        if x[0] == 'l':
            w.move_to(poses['left'], v_des, a_des)
        if x[0] == 'r':
            w.move_to(poses['right'], v_des, a_des)
        if x[0] == 'z':
            w.move_to(poses['zero'], v_des, a_des)
    else:
        w.pretty_print()

try:
    while True:
        try:
            step_interaction()
        except (ValueError):
            print('Bad input...')
        w.pull_status()
except (ThreadServiceExit, KeyboardInterrupt):
    w.stop()




#!/usr/bin/env python
from __future__ import print_function
import stretch_body.robot_params
stretch_body.robot_params.RobotParams.set_logging_level("DEBUG")
import sys
import stretch_body.stretch_gripper as gripper
import argparse
import stretch_body.hello_utils as hu
hu.print_stretch_re_use()

parser=argparse.ArgumentParser(description='Jog the griper from the keyboard')
args=parser.parse_args()

g=gripper.StretchGripper()
if not g.startup(threaded=False):
    exit()
g.pull_status()
v_des=g.params['motion']['default']['vel']
a_des=g.params['motion']['default']['accel']

def menu_top():
    print('------ MENU -------')
    print('m: menu')
    print('h: home')
    print('x: close by 10')
    print('y: open by 10')
    print('p: go to position (%6.2f to -100)'%g.pct_max_open)
    print('r: reboot')
    print('-----')
    print('a: open')
    print('b: zero')
    print('c: close')
    print('-----')
    print('1: speed slow')
    print('2: speed default')
    print('3: speed fast')
    print('4: speed max')
    print('-------------------')

def step_interaction():
    global v_des, a_des
    menu_top()
    x=sys.stdin.readline()
    if len(x)>1:
        if x[0]=='m':
            menu_top()
        if x[0]=='h':
            g.home()
        if x[0]=='x':
            g.move_by(-10.0, v_des, a_des)
        if x[0]=='y':
            g.move_by(10.0, v_des, a_des)
        if x[0]=='p':
            ff = int(sys.stdin.readline())
            ff=min(max(-100,ff),g.pct_max_open)
            g.move_to(ff, v_des, a_des)
        if x[0] == 'a':
            g.move_to(g.poses['open'], v_des, a_des)
        if x[0] == 'b':
            g.move_to(g.poses['zero'], v_des, a_des)
        if x[0] == 'c':
            g.move_to(g.poses['close'], v_des, a_des)
        if x[0]=='r':
            g.reboot()
            
        if x[0] == '1':
            v_des = g.params['motion']['slow']['vel']
            a_des = g.params['motion']['slow']['accel']

        if x[0] == '2':
            v_des = g.params['motion']['default']['vel']
            a_des = g.params['motion']['default']['accel']

        if x[0] == '3':
            v_des = g.params['motion']['fast']['vel']
            a_des = g.params['motion']['fast']['accel']

        if x[0] == '4':
            v_des = g.params['motion']['max']['vel']
            a_des = g.params['motion']['max']['accel']
    else:
        g.pretty_print()





try:
    while True:
        try:
            step_interaction()
        except (ValueError):
            print('Bad input...')
        g.pull_status()
except (KeyboardInterrupt):
    g.stop()




#!/usr/bin/env python
from __future__ import print_function
from stretch_body.robot import Robot
from stretch_body.hello_utils import *
import argparse
print_stretch_re_use()

parser=argparse.ArgumentParser(description='Run the Robot Monitor and print to console')
args=parser.parse_args()

r=Robot()
r.params['log_to_console']=1
print('Starting Robot Monitor. Ctrl-C to exit')
r.startup()


try:
    while True:
        time.sleep(1.0)
except (KeyboardInterrupt, SystemExit,ThreadServiceExit):
    pass
r.stop()



#!/usr/bin/env python

import stretch_body.hello_utils as hello_utils
import importlib
import stretch_body.robot_params

"""
Print out the robot parameters to the command line, 
including where each parameter value originates from
"""
def build_rep(prefix, params, origin,rep):
    for k in params.keys():
        np = prefix + '.' + str(k)
        if type(params[k]) == dict:
            rep.update(build_rep(np, params[k], origin,rep))
        else:
            rep[np]=[params[k], origin]
    return rep

rp=stretch_body.robot_params.RobotParams()

user_origin='stretch_user_params.yaml'
user_params = hello_utils.read_fleet_yaml(user_origin)

rep={}
build_rep('param',stretch_body.robot_params.nominal_system_params,'stretch_body.robot_params.nominal_system_params',rep)
build_rep('param',rp._nominal_params,'stretch_body.robot_params.nominal_params',rep)

for outside_params_origin in rp._user_params.get('params', []):
    outside_params=getattr(importlib.import_module(outside_params_origin),'params')
    build_rep('param', outside_params, outside_params_origin, rep)

config_origin='stretch_configuration_params.yaml'
config_params=hello_utils.read_fleet_yaml(config_origin)
build_rep('param',config_params,config_origin,rep)

build_rep('param',user_params,user_origin,rep)

print('#'*60+ ' Parameters for %s '%hello_utils.get_fleet_id()+'#'*60)
print ("{:<70} {:<70} {:<30}".format('Origin','Parameter','Value'))
print('-'*170)
for param in rep.keys():
    print("{:<70} {:<70} {:<30}".format(str(rep[param][1]), param, str(rep[param][0] )))



#!/usr/bin/env python
from __future__ import print_function
from stretch_body.dynamixel_XL430 import *
import argparse
import stretch_body.hello_utils as hu
hu.print_stretch_re_use()

parser=argparse.ArgumentParser(description='Reboot all Dynamixel servos on robot')
args=parser.parse_args()

print('---- Rebooting Head ---- ')
for id in range(25):
    m = DynamixelXL430(id, '/dev/hello-dynamixel-head')
    m.startup()
    if (m.do_ping(verbose=False)):
        m.do_reboot()
    else:
        m.stop()

print('---- Rebooting Wrist ---- ')
for id in range(25):
    m = DynamixelXL430(id, '/dev/hello-dynamixel-wrist')
    m.startup()
    if (m.do_ping(verbose=False)):
        m.do_reboot()
    else:
        m.stop()


#!/usr/bin/env python
from __future__ import print_function
import stretch_body.wrist_yaw as wrist_yaw
import argparse
import stretch_body.hello_utils as hu
hu.print_stretch_re_use()

parser=argparse.ArgumentParser(description='Calibrate the wrist_yaw position by moving to both hardstops')
args=parser.parse_args()

g=wrist_yaw.WristYaw()
if not g.startup(threaded=False):
    exit()
g.home()
g.stop()



#!/usr/bin/env python
from __future__ import print_function
import sys
from stretch_body.robot import Robot
from stretch_body.hello_utils import *
import argparse
print_stretch_re_use()

parser=argparse.ArgumentParser(description='Print all robot data to console')
args=parser.parse_args()

r=Robot()
r.startup()


def menu():
    print('------ MENU -------')
    print('m: menu')


def step_interaction():
    menu()
    x=sys.stdin.readline()
    if len(x)>1:
        if x[0]=='m':
            menu()

    else:
        r.pretty_print()

try:
    while True:
        try:
            step_interaction()
        except (ValueError):
            print('Bad input...')
except (KeyboardInterrupt, SystemExit,ThreadServiceExit):
    r.stop()



#!/usr/bin/env python
from __future__ import print_function
import stretch_body.xbox_controller as xc
import stretch_body.robot as rb
from stretch_body.hello_utils import *
import os
import time
import argparse
import lsb_release
print_stretch_re_use()

parser=argparse.ArgumentParser(description=
     'Jog the robot from an XBox Controller  \n' +
     '-------------------------------------\n' +
    'Left Stick X:\t Rotate base \n' +
    'Left Stick Y:\t Translate base \n' +
    'Right Trigger:\t Fast base motion \n' +
    'Right Stick X:\t Translate arm \n' +
    'Right Stick Y:\t Translate lift \n' +
    'Left Button:\t Rotate wrist CCW \n' +
    'Right Button:\t Rotate wrist CW \n' +
    'A/B Buttons:\t Close/Open gripper \n' +
    'Left/Right Pad:\t Head Pan \n' +
    'Top/Bottom Pad:\t Head tilt \n' +
    'Y Button :\t Go to stow position \n ' +
    'Start Button:\t Home robot \n ' +
    'Back Button (2 sec):\t Shutdown computer \n ' +
    '-------------------------------------\n',formatter_class=argparse.RawTextHelpFormatter)


args=parser.parse_args()


class CommandToLinearMotion():
    def __init__(self, command_dead_zone_value, move_duration_s, max_distance_m, accel_m):
        # This expects
        # a command value with a magnitude between 0.0 and 1.0, inclusive
        # a dead_zone with a magnitude greater than or equal to 0.0 and less than 1.0

        self.dead_zone = abs(command_dead_zone_value)
        self.move_duration_s = abs(move_duration_s)
        self.max_distance_m = abs(max_distance_m)
        self.accel_m = abs(accel_m)

        # check that the values are reasonable
        assert self.dead_zone >= 0.0
        assert self.dead_zone <= 0.9, 'WARNING: CommandToLinearMotion.__init__ command_dead_zone_value is strangely large command_dead_zone_value = abs({0}) > 0.9.'.format(
            command_dead_zone_value)
        assert self.move_duration_s > 0.01, 'WARNING: CommandToLinearMotion.__init__ move_duration_s = abs({0}) <= 0.01 seconds, which is a short time for a single move.'.format(
            move_duration_s)
        assert self.move_duration_s <= 1.0, 'WARNING: CommandToLinearMotion.__init__ move_duration_s = abs({0}) > 1.0 seconds, which is a long time for a single move.'.format(
            move_duration_s)
        assert self.max_distance_m <= 0.3, 'WARNING: CommandToLinearMotion.__init__ max_distance_m = abs({0}) > 0.3 meters, which is a long distance for a single move.'.format(
            max_distance_m)
        assert self.accel_m <= 30.0, 'WARNING: CommandToLinearMotion.__init__ accel_m = abs({0}) > 30.0 m/s^2, which is very high (> 3 g).'.format(
            accel_m)

    def get_dist_vel_accel(self, output_sign, command_value):
        # Larger commands attempt to move over larger distances in the
        # same amount of time by moving at higher velocities.
        c_val = abs(command_value)

        assert c_val <= 1.0, 'ERROR: CommandToLinearMotion.get_dist_vel_accel given command value > 1.0, command_value = {0}'.format(
            command_value)
        assert c_val > self.dead_zone, 'ERROR: CommandToLinearMotion.get_dist_vel_accel the command should not be executed due to its value being within the dead zone: abs(command_value) = abs({0}) <= {1} = self.dead_zone'.format(
            command_value, self.dead_zone)
        if 1:
            scale = (c_val - self.dead_zone) / (1.0 - self.dead_zone) ** 2
        else:
            scale = c_val - self.dead_zone
        d_m = (scale * (self.max_distance_m / (1.0 - self.dead_zone)))
        d_m = math.copysign(d_m, output_sign)
        v_m = d_m / self.move_duration_s  # average m/s for a move of distance d_m to last for time move_s
        a_m = self.accel_m
        return d_m, v_m, a_m


class CommandToRotaryMotion():
    def __init__(self, command_dead_zone_value, move_duration_s, max_angle_rad, accel_rad):
        # This expects
        # a command value with a magnitude between 0.0 and 1.0, inclusive
        # a dead_zone with a magnitude greater than or equal to 0.0 and less than 1.0

        self.dead_zone = abs(command_dead_zone_value)
        self.move_duration_s = abs(move_duration_s)
        self.max_angle_rad = abs(max_angle_rad)
        self.accel_rad = abs(accel_rad)

        # check that the values are reasonable
        assert self.dead_zone >= 0.0
        assert self.dead_zone <= 0.9, 'WARNING: CommandToRotaryMotion.__init__ command_dead_zone_value is strangely large command_dead_zone_value = abs({0}) > 0.9.'.format(
            command_dead_zone_value)
        assert self.move_duration_s > 0.01, 'WARNING: CommandToRotaryMotion.__init__ move_duration_s = abs({0}) <= 0.01 second, which is a short time for a single move.'.format(
            move_duration_s)
        assert self.move_duration_s <= 1.0, 'WARNING: CommandToRotaryMotion.__init__ move_duration_s = abs({0}) > 1.0 second, which is a long time for a single move.'.format(
            move_duration_s)
        assert self.max_angle_rad <= 0.7, 'WARNING: CommandToRotaryMotion.__init__ max_angle_rad = abs({0}) > 0.7 , which is a large angle for a single move (~40.0 deg).'.format(
            max_angle_rad)
        assert self.accel_rad <= 4.0*10, 'WARNING: CommandToRotaryMotion.__init__ accel_rad = abs({0}) > 4.0 rad/s^2, which is high.'.format(
            accel_rad)

    def get_dist_vel_accel(self, output_sign, command_value):
        # Larger commands attempt to move over larger distances in the
        # same amount of time by moving at higher velocities.
        c_val = abs(command_value)
        assert c_val <= 1.0, 'ERROR: CommandToRotaryMotion.get_dist_vel_accel given command value > 1.0, command_value = {0}'.format(
            command_value)
        assert c_val > self.dead_zone, 'ERROR: CommandToRotaryMotion.get_dist_vel_accel the command should not be executed due to its value being within the dead zone: abs(command_value) = abs({0}) <= {1} = self.dead_zone'.format(
            command_value, self.dead_zone)

        scale = c_val - self.dead_zone
        d_r = (scale * (self.max_angle_rad / (1.0 - self.dead_zone)))
        d_r = math.copysign(d_r, output_sign)
        v_r = d_r / self.move_duration_s  # average m/s for a move of distance d_m to last for time move_s
        a_r = self.accel_rad
        return d_r, v_r, a_r

# ######################### HEAD ########################################
head_pan_target=0.0
head_tilt_target=0.0



def manage_head(robot,controller_state):
    global head_pan_target, head_tilt_target
    if not use_head_mapping:
        return
    head_pan_rad = deg_to_rad(40.0)  # 25.0 #40.0
    head_pan_accel = 14.0  # 15.0
    head_pan_vel = 7.0  # 6.0
    head_pan_slew_down = 0.15
    head_pan_slew_up = .15

    camera_pan_right = controller_state['right_pad_pressed']
    camera_pan_left = controller_state['left_pad_pressed']
    camera_tilt_up = controller_state['top_pad_pressed']
    camera_tilt_down = controller_state['bottom_pad_pressed']

    # Slew target to zero if no buttons pushed
    if not camera_pan_left and not camera_pan_right:
        if head_pan_target >= 0:
            head_pan_target = max(0, head_pan_target - head_pan_slew_down)
        else:
            head_pan_target = min(0, head_pan_target + head_pan_slew_down)
    # Or slew up to max
    if camera_pan_right:
        head_pan_target = max(head_pan_target - head_pan_slew_up, -head_pan_rad)
    if camera_pan_left:
        head_pan_target = min(head_pan_target + head_pan_slew_up, head_pan_rad)
    robot.head.move_by('head_pan', head_pan_target, head_pan_vel, head_pan_accel)

    # Head tilt
    head_tilt_rad = deg_to_rad(40.0)  # 25.0 #40.0
    head_tilt_accel = 14.0  # 12.0  # 15.0
    head_tilt_vel = 7.0  # 10.0  # 6.0
    head_tilt_slew_down = 0.15
    head_tilt_slew_up = .15
    # Slew target to zero if no buttons pushed
    if not camera_tilt_up and not camera_tilt_down:
        if head_tilt_target >= 0:
            head_tilt_target = max(0, head_tilt_target - head_tilt_slew_down)
        else:
            head_tilt_target = min(0, head_tilt_target + head_tilt_slew_down)
    # Or slew up to max
    if camera_tilt_down:
        head_tilt_target = max(head_tilt_target - head_tilt_slew_up, -head_tilt_rad)
    if camera_tilt_up:
        head_tilt_target = min(head_tilt_target + head_tilt_slew_up, head_tilt_rad)
    robot.head.move_by('head_tilt', head_tilt_target, head_tilt_vel, head_tilt_accel)



# ######################### BASE ########################################
############################
# Regular Motion
dead_zone = 0.1  # 0.25 #0.1 #0.2 #0.3 #0.4
move_s = 0.6
max_dist_m = 0.06  # 0.04 #0.05
accel_m = 0.2  # 0.1
command_to_linear_motion = CommandToLinearMotion(dead_zone, move_s, max_dist_m, accel_m)

move_s = 0.05
max_dist_rad = 0.10  # 0.2 #0.25 #0.1 #0.09
accel_rad = 0.8  # 0.05
command_to_rotary_motion = CommandToRotaryMotion(dead_zone, move_s, max_dist_rad, accel_rad)
############################
# Fast Motion
fast_move_s = 0.6
fast_max_dist_m = 0.12
fast_accel_m = 0.8
# fast, but unstable on thresholds: 0.6 s, 0.15 m, 0.8 m/s^2

fast_command_to_linear_motion = CommandToLinearMotion(dead_zone, fast_move_s, fast_max_dist_m, fast_accel_m)
fast_move_s = 0.2
fast_max_dist_rad = 0.6
fast_accel_rad = 0.8
fast_command_to_rotary_motion = CommandToRotaryMotion(dead_zone, fast_move_s, fast_max_dist_rad, fast_accel_rad)
############################

def manage_base(robot,controller_state):
    forward_command = controller_state['left_stick_y']
    turn_command = controller_state['left_stick_x']

    fast_navigation_mode = False
    navigation_mode_trigger = controller_state['right_trigger_pulled']
    if (navigation_mode_trigger > 0.5):
        fast_navigation_mode = True

    ##################
    # convert robot commands to robot movement
    # only allow a pure translation or a pure rotation command
    if abs(forward_command) > abs(turn_command):
        if abs(forward_command) > dead_zone:
            output_sign = math.copysign(1, forward_command)
            if not fast_navigation_mode:
                d_m, v_m, a_m = command_to_linear_motion.get_dist_vel_accel(output_sign, forward_command)
            else:
                d_m, v_m, a_m = fast_command_to_linear_motion.get_dist_vel_accel(output_sign, forward_command)
            robot.base.translate_by(d_m, v_m, a_m)
    else:
        if abs(turn_command) > dead_zone:
            output_sign = -math.copysign(1, turn_command)
            if not fast_navigation_mode:
                d_rad, v_rad, a_rad = command_to_rotary_motion.get_dist_vel_accel(output_sign, turn_command)
            else:
                d_rad, v_rad, a_rad = fast_command_to_rotary_motion.get_dist_vel_accel(output_sign, turn_command)
            robot.base.rotate_by(d_rad, v_rad, a_rad)


# ######################### LIFT & ARM  ########################################

def manage_lift_arm(robot,controller_state):
    lift_command = controller_state['right_stick_y']
    arm_command = controller_state['right_stick_x']

    if abs(lift_command) > dead_zone:
        output_sign = math.copysign(1, lift_command)
        d_m, v_m, a_m = command_to_linear_motion.get_dist_vel_accel(output_sign, lift_command)
        robot.lift.move_by(d_m, v_m, a_m)

    if abs(arm_command) > dead_zone:
        output_sign = math.copysign(1, arm_command)
        d_m, v_m, a_m = command_to_linear_motion.get_dist_vel_accel(output_sign, arm_command)
        robot.arm.move_by(d_m, v_m, a_m)

# ######################### END OF ARM  ########################################
wrist_yaw_target=0.0
wrist_roll_target=0.0
wrist_pitch_target=0.0

def manage_end_of_arm(robot,controller_state):
    global wrist_yaw_target, wrist_roll_target, wrist_pitch_target
    wrist_yaw_left = controller_state['left_shoulder_button_pressed']
    wrist_yaw_right = controller_state['right_shoulder_button_pressed']


    close_gripper = controller_state['bottom_button_pressed']
    open_gripper = controller_state['right_button_pressed']

    wrist_yaw_rotate_rad = deg_to_rad(25.0)  # 60.0 #10.0 #5.0
    wrist_yaw_accel = 15.0  # 25.0 #30.0 #15.0 #8.0 #15.0
    wrist_yaw_vel = 25.0  # 10.0 #6.0 #3.0 #6.0
    wrist_yaw_slew_down = 0.15
    wrist_yaw_slew_up = .02

    wrist_roll_rotate_rad = deg_to_rad(25.0)  # 60.0 #10.0 #5.0
    wrist_roll_accel = 15.0  # 25.0 #30.0 #15.0 #8.0 #15.0
    wrist_roll_vel = 25.0  # 10.0 #6.0 #3.0 #6.0
    wrist_roll_slew_down = 0.15
    wrist_roll_slew_up = .02

    wrist_pitch_rotate_rad = deg_to_rad(25.0)  # 60.0 #10.0 #5.0
    wrist_pitch_accel = 25.0  # 25.0 #30.0 #15.0 #8.0 #15.0
    wrist_pitch_vel = 25.0  # 10.0 #6.0 #3.0 #6.0
    wrist_pitch_slew_down = 0.15
    wrist_pitch_slew_up = .02

    # Slew target to zero if no buttons pushed
    if not wrist_yaw_left and not wrist_yaw_right:
        if wrist_yaw_target >= 0:
            wrist_yaw_target = max(0, wrist_yaw_target - wrist_yaw_slew_down)
        else:
            wrist_yaw_target = min(0, wrist_yaw_target + wrist_yaw_slew_down)
    # Or slew up to max
    if wrist_yaw_left:
        wrist_yaw_target = min(wrist_yaw_target +wrist_yaw_slew_up, wrist_yaw_rotate_rad)
    if wrist_yaw_right:
        wrist_yaw_target = max(wrist_yaw_target - wrist_yaw_slew_up, -wrist_yaw_rotate_rad)
    robot.end_of_arm.move_by('wrist_yaw', wrist_yaw_target, wrist_yaw_vel, wrist_yaw_accel)



    if use_dex_wrist_mapping:
        wrist_roll_cw = controller_state['right_pad_pressed']
        wrist_roll_ccw = controller_state['left_pad_pressed']
        wrist_pitch_down = controller_state['top_pad_pressed']
        wrist_pitch_up = controller_state['bottom_pad_pressed']

        if not wrist_roll_cw and not wrist_roll_ccw:
            if wrist_roll_target >= 0:
                wrist_roll_target = max(0, wrist_roll_target - wrist_roll_slew_down)
            else:
                wrist_roll_target = min(0, wrist_roll_target + wrist_roll_slew_down)
        if wrist_roll_cw:
            wrist_roll_target = max(wrist_roll_target - wrist_roll_slew_up, -wrist_roll_rotate_rad)
        if wrist_roll_ccw:
            wrist_roll_target = min(wrist_roll_target + wrist_roll_slew_up, wrist_roll_rotate_rad)

        if not wrist_pitch_up and not wrist_pitch_down:
            if wrist_pitch_target >= 0:
                wrist_pitch_target = max(0, wrist_pitch_target - wrist_pitch_slew_down)
            else:
                wrist_pitch_target = min(0, wrist_pitch_target + wrist_pitch_slew_down)
        if wrist_pitch_up:
            wrist_pitch_target = max(wrist_pitch_target - wrist_pitch_slew_up, -wrist_pitch_rotate_rad)
        if wrist_pitch_down:
            wrist_pitch_target = min(wrist_pitch_target + wrist_pitch_slew_up, wrist_pitch_rotate_rad)

        robot.end_of_arm.move_by('wrist_roll', wrist_roll_target, wrist_pitch_vel, wrist_pitch_accel)
        #print('WW',rad_to_deg(wrist_roll_target),robot.end_of_arm.motors['wrist_roll'].status['pos_ticks'])
        robot.end_of_arm.move_by('wrist_pitch', wrist_pitch_target, wrist_pitch_vel, wrist_pitch_accel)

    if use_stretch_gripper_mapping:
        gripper_rotate_pct = 10.0
        gripper_accel = robot.end_of_arm.motors['stretch_gripper'].params['motion']['max']['accel']
        gripper_vel = robot.end_of_arm.motors['stretch_gripper'].params['motion']['max']['vel']

        if open_gripper:
            robot.end_of_arm.move_by('stretch_gripper', gripper_rotate_pct, gripper_vel, gripper_accel)
        elif close_gripper:
            robot.end_of_arm.move_by('stretch_gripper', -gripper_rotate_pct, gripper_vel, gripper_accel)
# ######################### SHUTDOWN  ########################################
shutdown_pc=False
ts_shutdown_start=0
def manage_shutdown(robot,controller_state):
    global shutdown_pc, ts_shutdown_start
    if controller_state['select_button_pressed']:
        if not shutdown_pc:
            ts_shutdown_start = time.time()
            shutdown_pc = True
        if time.time() - ts_shutdown_start > 2.0:
            robot.pimu.trigger_beep()
            robot.stow()
            robot.stop()
            time.sleep(1.0)
            if lsb_release.get_distro_information()['RELEASE'] == '20.04':
                os.system('canberra-gtk-play -f /usr/share/sounds/ubuntu/stereo/desktop-logout.ogg')
            if lsb_release.get_distro_information()['RELEASE'] == '18.04':
                os.system('paplay --device=alsa_output.pci-0000_00_1f.3.analog-stereo /usr/share/sounds/ubuntu/stereo/desktop-logout.ogg')
            os.system('sudo shutdown now')  # sudoers should be set up to not need a password
    else:
        shutdown_pc = False

# ######################### STOW and CALIBRATION  ########################################
def manage_stow(robot,controller_state):
    stow_robot = controller_state['top_button_pressed']
    if stow_robot and robot.is_calibrated():
        # Reset motion params as fast for xbox
        v = robot.end_of_arm.motors['wrist_yaw'].params['motion']['default']['vel']
        a = robot.end_of_arm.motors['wrist_yaw'].params['motion']['default']['accel']
        robot.end_of_arm.motors['wrist_yaw'].set_motion_params(v, a)
        robot.stow()

first_home_warn=True
def manage_calibration(robot,controller_state):
    global first_home_warn
    calibrate_the_robot = controller_state['start_button_pressed']
    if calibrate_the_robot:
        print('begin calibrating the robot')
        robot.home()
        print('finished calibrating the robot')
    else:
        if first_home_warn:
            print('press the start button to calibrate the robot')
        else:
            first_home_warn = False
# ######################### MAIN ########################################
use_head_mapping=True
use_dex_wrist_mapping=False
use_stretch_gripper_mapping=True

def main():
    global use_head_mapping, use_dex_wrist_mapping, use_stretch_gripper_mapping
    xbox_controller = xc.XboxController()
    xbox_controller.start()
    robot = rb.Robot()
    try:
        robot.startup()
        print('Using key mapping for tool: %s'%robot.end_of_arm.name)
        if robot.end_of_arm.name=='tool_none':
            use_head_mapping=True
            use_stretch_gripper_mapping=False
            use_dex_wrist_mapping=False

        if robot.end_of_arm.name=='tool_stretch_gripper':
            use_head_mapping=True
            use_stretch_gripper_mapping=True
            use_dex_wrist_mapping=False

        if robot.end_of_arm.name=='tool_stretch_dex_wrist':
            use_head_mapping=False
            use_stretch_gripper_mapping=True
            use_dex_wrist_mapping=True

        robot.pimu.trigger_beep()
        robot.push_command()
        time.sleep(0.5)

        robot.pimu.trigger_beep()
        robot.push_command()
        time.sleep(0.5)

        while True:
            controller_state = xbox_controller.get_state()
            if not robot.is_calibrated():
                manage_calibration(robot, controller_state)
            else:
                manage_base(robot, controller_state)
                manage_lift_arm(robot, controller_state)
                manage_end_of_arm(robot, controller_state)
                manage_head(robot, controller_state)
                manage_stow(robot, controller_state)
            manage_shutdown(robot, controller_state)
            robot.push_command()
            time.sleep(0.05)
    except (ThreadServiceExit, KeyboardInterrupt, SystemExit):
        robot.stop()
        xbox_controller.stop()

if __name__ == "__main__":
    main()



#!/usr/bin/env python
from __future__ import print_function
import time
import stretch_body.pimu as pimu
from stretch_body.hello_utils import *
import argparse
print_stretch_re_use()
# ###################################


parser=argparse.ArgumentParser(description='Visualize Pimu (Power+IMU) board data with an oscilloscope')

group = parser.add_mutually_exclusive_group(required=True)
group.add_argument("--cliff", help="Scope base cliff sensors",action="store_true")
group.add_argument("--at_cliff", help="Scope base at_cliff signal",action="store_true")
group.add_argument("--voltage", help="Scope bus voltage (V)",action="store_true")
group.add_argument("--current", help="Scope bus current (A)",action="store_true")
group.add_argument("--temp", help="Scope base internal temperature (C)",action="store_true")
group.add_argument("--ax", help="Scope base accelerometer AX",action="store_true")
group.add_argument("--ay", help="Scope base accelerometer AY",action="store_true")
group.add_argument("--az", help="Scope base accelerometer AZ",action="store_true")
group.add_argument("--mx", help="Scope base magnetometer MX",action="store_true")
group.add_argument("--my", help="Scope base magnetometer MY",action="store_true")
group.add_argument("--mz", help="Scope base magnetometer MZ",action="store_true")
group.add_argument("--gx", help="Scope base gyro GX",action="store_true")
group.add_argument("--gy", help="Scope base gyro GY",action="store_true")
group.add_argument("--gz", help="Scope base gyro GZ",action="store_true")
group.add_argument("--roll", help="Scope base imu Roll",action="store_true")
group.add_argument("--pitch", help="Scope base imu Pitch",action="store_true")
group.add_argument("--heading", help="Scope base imu Heading",action="store_true")
group.add_argument("--bump", help="Scope base imu bump level",action="store_true")
args, _ = parser.parse_known_args()


p=pimu.Pimu()
if not p.startup():
    exit()


# Cliff
if args.cliff:
    import stretch_body.scope as scope
    s=scope.Scope4(yrange=[-75,75],title='Cliff')
    try:
        while True:
            p.pull_status()
            s.step_display(p.status['cliff_range'][0],p.status['cliff_range'][1],p.status['cliff_range'][2],p.status['cliff_range'][3])
            print(p.status['cliff_range'])
            time.sleep(0.02)
    except (SystemExit, KeyboardInterrupt):
        p.stop()

if args.at_cliff:
    import stretch_body.scope as scope
    s=scope.Scope4(yrange=[-.25,1.25],title='At Cliff')
    try:
        while True:
            p.pull_status()
            s.step_display(p.status['at_cliff'][0],p.status['at_cliff'][1],p.status['at_cliff'][2],p.status['at_cliff'][3])
            print(p.status['at_cliff'])
            time.sleep(0.02)
    except (SystemExit, KeyboardInterrupt):
        p.stop()

# Voltage
if args.voltage:
    import stretch_body.scope as scope
    s=scope.Scope(yrange=[9,15],title='Voltage (V)')
    try:
        while True:
            p.pull_status()
            s.step_display(p.status['voltage'])
            print(p.status['voltage'])
            time.sleep(0.02)
    except (SystemExit, KeyboardInterrupt):
        p.stop()

# Temp
if args.temp:
    import stretch_body.scope as scope
    s=scope.Scope(yrange=[0,70],title='Temp (C)')
    try:
        while True:
            p.pull_status()
            s.step_display(p.status['temp'])
            print(p.status['temp'])
            time.sleep(0.02)
    except (SystemExit, KeyboardInterrupt):
        p.stop()

# Current
if args.current:
    import stretch_body.scope as scope
    s=scope.Scope(yrange=[0,5],title='Current (A)')
    try:
        while True:
            p.pull_status()
            s.step_display(p.status['current'])
            print(p.status['current'])
            time.sleep(0.02)
    except (SystemExit, KeyboardInterrupt):
        p.stop()

    sp = scope.Scope(yrange=[-60,60], title='Pitch')
    sh = scope.Scope(yrange=[0,360], title='Heading')


#Mag
if args.bump:
    import stretch_body.scope as scope
    s = scope.Scope(yrange=[-1,15],title='Bump')
    itr=0
    try:
        while True:
            p.pull_status()
            time.sleep(0.02)
            itr=itr+1
            if itr==50:
                itr=0
                p.trigger_beep()
                p.push_command()
            print('Bump',p.status['imu']['bump'])
            s.step_display(p.status['imu']['bump'])
    except (SystemExit, KeyboardInterrupt):
        p.stop()

#Mag
if args.mx:
    import stretch_body.scope as scope
    s = scope.Scope(yrange=[-20,20],title='MX')
    try:
        while True:
            p.pull_status()
            time.sleep(0.02)
            print('MX',p.status['imu']['mx'])
            s.step_display(p.status['imu']['mx'])
    except (SystemExit, KeyboardInterrupt):
        p.stop()

if args.my:
    import stretch_body.scope as scope
    s = scope.Scope(yrange=[-20,20],title='MY')
    try:
        while True:
            p.pull_status()
            time.sleep(0.02)
            print('MY',p.status['imu']['my'])
            s.step_display(p.status['imu']['my'])
    except (SystemExit, KeyboardInterrupt):
        p.stop()

if args.mz:
    import stretch_body.scope as scope
    s = scope.Scope(yrange=[-20,20],title='MZ')
    try:
        while True:
            p.pull_status()
            time.sleep(0.02)
            print('MZ',p.status['imu']['mz'])
            s.step_display(p.status['imu']['mz'])
    except (SystemExit, KeyboardInterrupt):
        p.stop()

#Accel
if args.ax:
    import stretch_body.scope as scope
    s = scope.Scope(yrange=[-2,2], title='AX')
    try:
        while True:
            p.pull_status()
            time.sleep(0.02)
            print('AX',p.status['imu']['ax'])
            s.step_display(p.status['imu']['ax'])
    except (SystemExit, KeyboardInterrupt):
        p.stop()

if args.ay:
    import stretch_body.scope as scope
    s = scope.Scope(yrange=[-2,2], title='AY')
    try:
        while True:
            p.pull_status()
            time.sleep(0.02)
            print('AY',p.status['imu']['ay'])
            s.step_display(p.status['imu']['ay'])
    except (SystemExit, KeyboardInterrupt):
        p.stop()

if args.az:
    import stretch_body.scope as scope
    s = scope.Scope(yrange=[-11,-8], title='AZ')
    try:
        while True:
            p.pull_status()
            time.sleep(0.02)
            print('AZ',p.status['imu']['az'])
            s.step_display(p.status['imu']['az'])
    except (SystemExit, KeyboardInterrupt):
        p.stop()


if args.gx:
    import stretch_body.scope as scope
    s = scope.Scope(yrange=[-0.5,0.5], title='GX')
    try:
        while True:
            p.pull_status()
            time.sleep(0.02)
            print('GX',p.status['imu']['gx'])
            s.step_display(p.status['imu']['gx'])
    except (SystemExit, KeyboardInterrupt):
        p.stop()

if args.gy:
    import stretch_body.scope as scope
    s = scope.Scope(yrange=[-0.5,0.5], title='GY')
    try:
        while True:
            p.pull_status()
            time.sleep(0.02)
            print('GY',p.status['imu']['gy'])
            s.step_display(p.status['imu']['gy'])
    except (SystemExit, KeyboardInterrupt):
        p.stop()

if args.gz:
    import stretch_body.scope as scope
    s = scope.Scope(yrange=[-0.5,0.5], title='GZ')
    try:
        while True:
            p.pull_status()
            time.sleep(0.02)
            print('GZ',p.status['imu']['gz'])
            s.step_display(p.status['imu']['gz'])
    except (SystemExit, KeyboardInterrupt):
        p.stop()

# IMU
if args.roll:
    import stretch_body.scope as scope
    s = scope.Scope(title='Roll',yrange=[-20,20])
    try:
        while True:
            p.pull_status()
            time.sleep(0.02)
            print('Roll',rad_to_deg(p.status['imu']['roll']))
            s.step_display(rad_to_deg(p.status['imu']['roll']))
    except (SystemExit, KeyboardInterrupt):
        p.stop()

if args.pitch:
    import stretch_body.scope as scope
    s = scope.Scope(yrange=[-20,20], title='Pitch')
    try:
        while True:
            p.pull_status()
            time.sleep(0.02)
            print('Pitch',rad_to_deg(p.status['imu']['pitch']))
            s.step_display(rad_to_deg(p.status['imu']['pitch']))
    except (SystemExit, KeyboardInterrupt):
        p.stop()

if args.heading:
    import stretch_body.scope as scope
    s = scope.Scope(yrange=[-10,370], title='Heading')
    try:
        while True:
            p.pull_status()
            time.sleep(0.02)
            print('Heading',rad_to_deg(p.status['imu']['heading']))
            s.step_display(rad_to_deg(p.status['imu']['heading']))
    except (SystemExit, KeyboardInterrupt):
        p.stop()



#!/usr/bin/env python
from __future__ import print_function
import stretch_body.arm as arm
import stretch_body.hello_utils as hu
hu.print_stretch_re_use()

import argparse
parser=argparse.ArgumentParser(description='Calibrate arm position by moving to hardstop')
args=parser.parse_args()

a=arm.Arm()
if not a.startup(threaded=False):
    exit()
a.home()
a.stop()




#!/usr/bin/env python
from __future__ import print_function
import os
import yaml
import cv2

image_file = "/etc/hello-robot/stretch_about.png"
image = cv2.imread(image_file)
if image is None:
    print("Unable to find stretch_about.png in /etc/hello-robot")
    exit()

configuration_params_filename = os.environ['HELLO_FLEET_PATH']+'/'+os.environ['HELLO_FLEET_ID']+'/stretch_configuration_params.yaml'

with open(configuration_params_filename, 'r') as fid:
    configuration_params = yaml.safe_load(fid)
    robot_info = configuration_params['robot']
    batch_name = robot_info['batch_name']
    serial_number = robot_info['serial_no']
    model=robot_info['model_name']

# Write information on the image
font = cv2.FONT_HERSHEY_SIMPLEX #cv2.FONT_HERSHEY_PLAIN
font_scale = 0.7
line_color = [18, 40, 55]
line_width = 1

batch_name_string = 'Batch Name: {0}'.format(batch_name)
serial_number_string = 'Serial Number: {0}'.format(serial_number)
model_string = 'Model: {0}'.format(model)
color = (0, 0, 255)

# see the following page for a helpful reference
# https://stackoverflow.com/questions/51285616/opencvs-gettextsize-and-puttext-return-wrong-size-and-chop-letters-with-low

text_x = 310
text_y = 375

vertical_spacing_pix = 40

cv2.rectangle(image, (280, 333), (720, 443), (245, 241, 253), cv2.FILLED)
cv2.putText(image, batch_name_string, (text_x, text_y), font, font_scale, line_color, line_width, cv2.LINE_AA)
cv2.putText(image, serial_number_string, (text_x, text_y + vertical_spacing_pix), font, font_scale, line_color, line_width, cv2.LINE_AA)
cv2.putText(image, model_string, (text_x, text_y + vertical_spacing_pix*2), font, font_scale, line_color, line_width, cv2.LINE_AA)
window_name = 'S T R E T C H RESEARCH EDITION'
cv2.imshow(window_name, image)
# Used exiting advice found via the following link:
# https://stackoverflow.com/questions/35003476/opencv-python-how-to-detect-if-a-window-is-closed
while cv2.getWindowProperty(window_name, 0) >= 0:
    key_code = cv2.waitKey(1)
    if key_code > 0:
        # key pressed
        cv2.destroyAllWindows()
        break        
    elif cv2.getWindowProperty(window_name,cv2.WND_PROP_VISIBLE) < 1:
        # window closed
        break     



#!/usr/bin/env python
from __future__ import print_function
import sys, tty, termios
import stretch_body.robot as hello_robot
from stretch_body.hello_utils import *
import argparse
print_stretch_re_use()


parser=argparse.ArgumentParser(description='Control the robot base, lift, arm, head, and tool from the keyboard')
args=parser.parse_args()


robot=hello_robot.Robot()
robot.startup()

small_move_m=.01
large_move_m=0.1
small_rotate_rad=deg_to_rad(1.0)
large_rotate_rad=deg_to_rad(10.0)

small_lift_move_m=.01
large_lift_move_m=.075
small_arm_move_m=.010
large_arm_move_m=.075

gg=0.5

def get_keystroke():

    fd=sys.stdin.fileno()
    old_settings=termios.tcgetattr(fd)
    try:
        tty.setraw(sys.stdin.fileno())
        ch=sys.stdin.read(1)
    finally:
        termios.tcsetattr(fd,termios.TCSADRAIN,old_settings)
    return ch

def menu():
    print('--------------')
    print('Mobile Base')
    print('i / , / j / l : small forward / back / left / right')
    print('I / < / J / L : large forward / back / left / right')
    print('')
    print('Arm')
    print('w / x / a / d : small up / down / in / out')
    print('W / X / A / D : large up / down / in / out')
    print('')
    print('Head')
    print('f / h / t / b : ahead / back / tool / wheels')
    print('')
    print('Tool')
    print('8 / 2 / 0 / 5 : forward / stow / open / close')
    print('')
    print('Mechaduinos')
    print('e / p  : effort / pose ')
    print('')
    print('System')
    print('? / q : help / quit')
    print('--------------')

try:
    menu()
    while True:
        if True:

            c=get_keystroke()

            # mobile base commands
            if c == 'i':
                robot.base.translate_by(small_move_m)
            if c == ',':
                robot.base.translate_by(-1*small_move_m)
            if c == 'j':
                robot.base.rotate_by(small_rotate_rad)
            if c == 'l':
                robot.base.rotate_by(-1*small_rotate_rad)
            if c == 'I':
                robot.base.translate_by(large_move_m)
            if c == '<':
                robot.base.translate_by(-1*large_move_m)
            if c == 'J':
                robot.base.rotate_by(large_rotate_rad)
            if c == 'L':
                robot.base.rotate_by(-1*large_rotate_rad)

            # lift commands
            if c == 'w':
                robot.lift.move_by(small_lift_move_m)
            if c == 'x':
                robot.lift.move_by(-1*small_lift_move_m)
            if c == 'W':
                robot.lift.move_by(large_lift_move_m)
            if c == 'X':
                robot.lift.move_by(-1*large_lift_move_m)

            # arm commands

            if c == 'a':
                robot.arm.move_by(-1*small_arm_move_m)
            if c == 'd':
                robot.arm.move_by(small_arm_move_m)
            if c == 'A':
                robot.arm.move_by(-1 * large_arm_move_m)
            if c == 'D':
                robot.arm.move_by(large_arm_move_m)

            # head commands
            if c == 'f':
                robot.head.pose('ahead')
            if c == 'h':
                robot.head.pose('back')
            if c == 't':
                robot.head.pose('tool')
            if c == 'b':
                robot.head.pose('wheels')

            # tool commands
            if c == '2':
                robot.end_of_arm.pose('wrist_yaw','stow')
            if c == '8':
                robot.end_of_arm.pose('wrist_yaw','forward')
            if c == '0':
                robot.end_of_arm.pose('stretch_gripper','open')
            if c == '5':
                robot.end_of_arm.pose('stretch_gripper','close')
                
            # Mechaduino commands
            if c == 'e':
                print('Mobile Base Effort: ', robot.base.left_wheel.status['effort'],robot.base.right_wheel.status['effort'])
                print('Arm Force: ', robot.status['arm']['force'])
            if c == 'p':
                print('Mobile Base Pose: ', robot.base.status['pos'])
                print('Arm Position: ', robot.status['arm']['pos'])

            robot.push_command()

            
            # System commands
            if c=='Q' or c=='q':
                print('Quiting')
                robot.stop()
                sys.exit()
            if c=='?':
                menu()
                
except (ThreadServiceExit,KeyboardInterrupt, SystemExit):
    pass



#!/usr/bin/env python
from __future__ import print_function
import os
import stretch_body.hello_utils as hu
import lsb_release
hu.print_stretch_re_use()
os.system('pactl set-default-sink alsa_output.pci-0000_00_1f.3.analog-stereo')
if lsb_release.get_distro_information()['RELEASE']=='20.04':
    os.system('amixer set Master 100%')
    os.system('canberra-gtk-play -f /usr/share/sounds/ubuntu/stereo/desktop-login.ogg')
if lsb_release.get_distro_information()['RELEASE']=='18.04':
    os.system('amixer set Master 80%')
    os.system('paplay --device=alsa_output.pci-0000_00_1f.3.analog-stereo /usr/share/sounds/ubuntu/stereo/desktop-login.ogg')




#!/usr/bin/env python
from __future__ import print_function
from rplidar import *
import argparse
import stretch_body.hello_utils as hu
hu.print_stretch_re_use()

parser=argparse.ArgumentParser(description='Tool to control the RP-Lidar')
group = parser.add_mutually_exclusive_group(required=True)
group.add_argument("--motor_on", help="Turn motor on",action="store_true")
group.add_argument("--motor_off", help="Turn motor off",action="store_true")
group.add_argument("--info", help="Device info",action="store_true")
group.add_argument("--health", help="Get device health",action="store_true")
group.add_argument("--reset", help="Reset device",action="store_true")
group.add_argument("--range", help="Print range reading",action="store_true")
args=parser.parse_args()

try:
    lidar = RPLidar('/dev/hello-lrf')
except RPLidarException:
    print ('RPLidar not present')
    exit()
prev_motor_running=lidar.motor_running

if args.info:
    print(lidar.get_info())

if args.motor_on:
    lidar.start_motor()
    prev_motor_running=True

if args.motor_off:
    lidar.stop_motor()
    prev_motor_running=False

if args.health:
    print(lidar.get_health())

if args.reset:
    print(lidar.reset())

if args.range:
    for i, scan in enumerate(lidar.iter_scans()):
        if i > 0:
            break
        print('%d: Got %d measurments' % (i, len(scan)))
        print('Scan',scan)


if not args.motor_on: #Turn off motor by default
    lidar.stop_motor()





#!/usr/bin/env python
from __future__ import print_function
import stretch_body.robot as rb
import stretch_body.hello_utils as hu
hu.print_stretch_re_use()

robot = rb.Robot()
robot.startup()
if not robot.pimu.status['runstop_event']:
    robot.stow()
else:
    robot.logger.warning('Cannot stow while run-stopped')
robot.stop()



import setuptools
from os import listdir
from os.path import isfile, join


with open("README.md", "r") as fh:
    long_description = fh.read()

script_path='./bin'
stretch_scripts={script_path+'/'+f for f in listdir(script_path) if isfile(join(script_path, f))}

setuptools.setup(
    name="hello_robot_stretch_body_tools",
    version="0.3.3.dev1",
    author="Hello Robot Inc",
    author_email="support@hello-robot.com",
    description="Stretch Body Tools",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/hello-robot/stretch_body",
    scripts = stretch_scripts,
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "Operating System :: OS Independent",
        "License :: OSI Approved :: Apache Software License"
    ],
    install_requires=['numpy', 'scipy', 'matplotlib', 'ipython', 'pandas', 'sympy', 'nose',
                      'inputs', 'drawnow', 'rplidar-roboticia', 'snakeviz', 'pyusb', 'SpeechRecognition', 'pixel-ring',
                      'click', 'cma', 'opencv-contrib-python', 'colorama', 'scikit-image', 'open3d', 'pyrealsense2',
                      'pyglet == 1.4.10; python_version < "3.0"', 'trimesh==3.6.38', 'urdfpy', # required for urdfpy viz in py2
                      'pyyaml>=5.1', # required for yaml.FullLoader
                      'hello-robot-stretch-body>=0.3.5dev1']
)

#classifiers = [
#    "Programming Language :: Python :: 2",
#    "License :: OSI Approved :: BSD License",
#    "Operating System :: OS Independent",



#!/usr/bin/env python
from __future__ import print_function
import sys
from stretch_body.pimu import Pimu
import stretch_body.hello_utils as hu
hu.print_stretch_re_use()

p=Pimu()
if not p.startup():
    exit()

import argparse
parser=argparse.ArgumentParser(description='Comnmand and query the Pimu (Power+IMU) board from the keyboard')
args=parser.parse_args()

def menu():
    print('------ MENU -------')
    print('m: menu')
    print('i: reset imu')
    print('f: toggle fan')
    print('b: toggle buzzer')
    print('p: beep')
    print('t: trigger motor sync')
    print('r: reset board')
    print('x: reset runstop event')
    print('o: trigger runstop event')
    print('l: trigger lightbar test event')
    print('y: reset cliff event')
    print('-------------------')

def step_interaction():
    menu()
    x=sys.stdin.readline()
    p.pull_status()
    if len(x)>1:
        if x[0]=='m':
            menu()
        if x[0]=='x':
            print('Resetting Runstop Event')
            p.runstop_event_reset()
        if x[0]=='l':
            print('Starting lightbar test')
            p.trigger_lightbar_test()
        if x[0]=='o':
            print('Triggering Runstop Event')
            p.runstop_event_trigger()
        if x[0]=='y':
            print('Resetting Cliff Event')
            p.cliff_event_reset()
        if x[0]=='r':
            print('Resetting Board!!!')
            p.board_reset()
        if x[0]=='i':
            p.imu_reset()
        if x[0]=='f':
            if p.status['fan_on']:
                p.set_fan_off()
            else:
                p.set_fan_on()
        if x[0] == 'p':
            p.trigger_beep()
        if x[0]=='b':
            if p.status['buzzer_on']:
                p.set_buzzer_off()
            else:
                p.set_buzzer_on()
        if x[0] == 't':
            p.trigger_motor_sync()
        p.push_command()
    else:
        p.pretty_print()

try:
    while True:
        try:
            step_interaction()
        except (ValueError):
            print('Bad input...')
except (KeyboardInterrupt, SystemExit):
    p.stop()



#!/usr/bin/env python
from __future__ import print_function
import stretch_body.stretch_gripper as gripper
import time
import argparse
import stretch_body.hello_utils as hu
hu.print_stretch_re_use()

parser=argparse.ArgumentParser(description='Calibrate the gripper position by closing until motion stops')
args=parser.parse_args()

g=gripper.StretchGripper()
if not g.startup(threaded=False):
    exit()
g.home()
time.sleep(3.0)
g.stop()